import glob
import os
import textwrap

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient
from conans.util.files import load


def get_requires_from_content(content):
    for line in content.splitlines():
        if "Requires:" in line:
            return line
    return ""


def test_pkg_config_dirs():
    # https://github.com/conan-io/conan/issues/2756
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            name = "MyLib"
            version = "0.1"

            def package_info(self):
                self.cpp_info.frameworkdirs = []
                self.cpp_info.filter_empty = False
                libname = "mylib"
                fake_dir = os.path.join("/", "my_absoulte_path", "fake")
                include_dir = os.path.join(fake_dir, libname, "include")
                lib_dir = os.path.join(fake_dir, libname, "lib")
                lib_dir2 = os.path.join(self.package_folder, "lib2")
                self.cpp_info.includedirs = [include_dir]
                self.cpp_info.libdirs = [lib_dir, lib_dir2]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install MyLib/0.1@ -g PkgConfigDeps")

    pc_path = os.path.join(client.current_folder, "MyLib.pc")
    assert os.path.exists(pc_path) is True
    pc_content = load(pc_path)

    assert 'Name: MyLib' in pc_content
    assert 'Description: Conan package: MyLib' in pc_content
    assert 'Version: 0.1' in pc_content
    assert 'Libs: -L"${libdir1}" -L"${libdir2}"' in pc_content
    assert 'Cflags: -I"${includedir1}"' in pc_content

    def assert_is_abs(path):
        assert os.path.isabs(path) is True

    for line in pc_content.splitlines():
        if line.startswith("includedir1="):
            assert_is_abs(line[len("includedir1="):])
            assert line.endswith("include")
        elif line.startswith("libdir1="):
            assert_is_abs(line[len("libdir1="):])
            assert line.endswith("lib")
        elif line.startswith("libdir2="):
            assert "${prefix}/lib2" in line


def test_empty_dirs():
    # Adding in package_info all the empty directories
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            name = "MyLib"
            version = "0.1"

            def package_info(self):
                self.cpp_info.includedirs = []
                self.cpp_info.libdirs = []
                self.cpp_info.bindirs = []
                self.cpp_info.libs = []
                self.cpp_info.frameworkdirs = []
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install MyLib/0.1@ -g PkgConfigDeps")

    pc_path = os.path.join(client.current_folder, "MyLib.pc")
    assert os.path.exists(pc_path) is True
    pc_content = load(pc_path)
    expected = textwrap.dedent("""
        Name: MyLib
        Description: Conan package: MyLib
        Version: 0.1
        Libs:%s
        Cflags: """ % " ")  # ugly hack for trailing whitespace removed by IDEs
    assert "\n".join(pc_content.splitlines()[1:]) == expected


def test_system_libs():
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conans.tools import save
        import os

        class PkgConfigConan(ConanFile):
            name = "MyLib"
            version = "0.1"

            def package(self):
                save(os.path.join(self.package_folder, "lib", "file"), "")

            def package_info(self):
                self.cpp_info.libs = ["mylib1", "mylib2"]
                self.cpp_info.system_libs = ["system_lib1", "system_lib2"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.run("install MyLib/0.1@ -g PkgConfigDeps")

    pc_content = client.load("MyLib.pc")
    assert 'Libs: -L"${libdir1}" -lmylib1 -lmylib2 -lsystem_lib1 -lsystem_lib2' in pc_content


def test_multiple_include():
    # https://github.com/conan-io/conan/issues/7056
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conans.tools import save
        import os

        class PkgConfigConan(ConanFile):
            def package(self):
                for p in ["inc1", "inc2", "inc3/foo", "lib1", "lib2"]:
                    save(os.path.join(self.package_folder, p, "file"), "")

            def package_info(self):
                self.cpp_info.includedirs = ["inc1", "inc2", "inc3/foo"]
                self.cpp_info.libdirs = ["lib1", "lib2"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . pkg/0.1@")
    client.run("install pkg/0.1@ -g PkgConfigDeps")

    pc_content = client.load("pkg.pc")
    assert "includedir1=${prefix}/inc1" in pc_content
    assert "includedir2=${prefix}/inc2" in pc_content
    assert "includedir3=${prefix}/inc3/foo" in pc_content
    assert "libdir1=${prefix}/lib1" in pc_content
    assert "libdir2=${prefix}/lib2" in pc_content
    assert 'Libs: -L"${libdir1}" -L"${libdir2}"' in pc_content
    assert 'Cflags: -I"${includedir1}" -I"${includedir2}" -I"${includedir3}"' in pc_content


def test_custom_content():
    # https://github.com/conan-io/conan/issues/7661
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conans.tools import save
        import os
        import textwrap

        class PkgConfigConan(ConanFile):

            def package(self):
                save(os.path.join(self.package_folder, "include" ,"file"), "")
                save(os.path.join(self.package_folder, "lib" ,"file"), "")

            def package_info(self):
                custom_content = textwrap.dedent(\"""
                        datadir=${prefix}/share
                        schemasdir=${datadir}/mylib/schemas
                        bindir=${prefix}/bin
                    \""")
                self.cpp_info.set_property("pkg_config_custom_content", custom_content)
                self.cpp_info.includedirs = ["include"]
                self.cpp_info.libdirs = ["lib"]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . pkg/0.1@")
    client.run("install pkg/0.1@ -g PkgConfigDeps")

    pc_content = client.load("pkg.pc")
    assert "libdir1=${prefix}/lib" in pc_content
    assert "datadir=${prefix}/share" in pc_content
    assert "schemasdir=${datadir}/mylib/schemas" in pc_content
    assert "bindir=${prefix}/bin" in pc_content
    assert "Name: pkg" in pc_content


def test_custom_content_and_version_components():
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conans.tools import save
        import os
        import textwrap

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.components["mycomponent"].set_property("pkg_config_custom_content",
                                                                     "componentdir=${prefix}/mydir")
                self.cpp_info.components["mycomponent"].set_property("component_version",
                                                                     "19.8.199")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . pkg/0.1@")
    client.run("install pkg/0.1@ -g PkgConfigDeps")
    pc_content = client.load("pkg-mycomponent.pc")
    assert "componentdir=${prefix}/mydir" in pc_content
    assert "Version: 19.8.199" in pc_content


def test_pkg_with_public_deps_and_component_requires():
    """
    Testing a complex structure like:

    * first/0.1
        - Global pkg_config_name == "myfirstlib"
        - Components: "cmp1"
    * other/0.1
    * second/0.1
        - Requires: "first/0.1"
        - Components: "mycomponent", "myfirstcomp"
            + "mycomponent" requires "first::cmp1"
            + "myfirstcomp" requires "mycomponent"
    * third/0.1
        - Requires: "second/0.1", "other/0.1"

    Expected file structure after running PkgConfigDeps as generator:
        - other.pc
        - myfirstlib-cmp1.pc
        - myfirstlib.pc
        - second-mycomponent.pc
        - second-myfirstcomp.pc
        - second.pc
        - third.pc
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "myfirstlib")
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . first/0.1@")
    client.save({"conanfile.py": GenConanfile("other", "0.1").with_package_file("file.h", "0.1")})
    client.run("create .")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "first/0.1"

            def package_info(self):
                self.cpp_info.components["mycomponent"].requires.append("first::cmp1")
                self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . second/0.1@")
    client.save({"conanfile.py": GenConanfile("third", "0.1").with_package_file("file.h", "0.1")
                                                             .with_require("second/0.1")
                                                             .with_require("other/0.1")},
                clean_first=True)
    client.run("create .")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        third/0.1

        [generators]
        PkgConfigDeps
        """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")

    pc_content = client2.load("third.pc")
    # Originally posted: https://github.com/conan-io/conan/issues/9939
    assert "Requires: second other" == get_requires_from_content(pc_content)
    pc_content = client2.load("second.pc")
    assert "Requires: second-mycomponent second-myfirstcomp" == get_requires_from_content(pc_content)
    pc_content = client2.load("second-mycomponent.pc")
    assert "Requires: myfirstlib-cmp1" == get_requires_from_content(pc_content)
    pc_content = client2.load("second-myfirstcomp.pc")
    assert "Requires: second-mycomponent" == get_requires_from_content(pc_content)
    pc_content = client2.load("myfirstlib.pc")
    assert "Requires: myfirstlib-cmp1" == get_requires_from_content(pc_content)
    pc_content = client2.load("other.pc")
    assert "" == get_requires_from_content(pc_content)


def test_pkg_with_public_deps_and_component_requires_2():
    """
    Testing another complex structure like:

    * other/0.1
        - Global pkg_config_name == "fancy_name"
        - Components: "cmp1", "cmp2", "cmp3"
            + "cmp1" pkg_config_name == "component1" (it shouldn't be affected by "fancy_name")
            + "cmp3" pkg_config_name == "component3" (it shouldn't be affected by "fancy_name")
            + "cmp3" requires "cmp1"
    * pkg/0.1
        - Requires: "other/0.1" -> "other::cmp1"

    Expected file structure after running PkgConfigDeps as generator:
        - component1.pc
        - component3.pc
        - other-cmp2.pc
        - other.pc
        - pkg.pc
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "fancy_name")
                self.cpp_info.components["cmp1"].libs = ["other_cmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "component1")
                self.cpp_info.components["cmp2"].libs = ["other_cmp2"]
                self.cpp_info.components["cmp3"].requires.append("cmp1")
                self.cpp_info.components["cmp3"].set_property("pkg_config_name", "component3")
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . other/1.0@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "other/1.0"

            def package_info(self):
                self.cpp_info.requires = ["other::cmp1"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . pkg/0.1@")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        pkg/0.1

        [generators]
        PkgConfigDeps
        """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")
    pc_content = client2.load("pkg.pc")
    assert "Requires: component1" == get_requires_from_content(pc_content)
    pc_content = client2.load("fancy_name.pc")
    assert "Requires: component1 fancy_name-cmp2 component3" == get_requires_from_content(pc_content)
    assert client2.load("component1.pc")
    assert client2.load("fancy_name-cmp2.pc")
    pc_content = client2.load("component3.pc")
    assert "Requires: component1" == get_requires_from_content(pc_content)


def test_pkg_config_name_full_aliases():
    """
    Testing a simpler structure but paying more attention into several aliases.
    Expected file structure after running PkgConfigDeps as generator:
        - compo1.pc
        - compo1_alias.pc
        - pkg_alias1.pc
        - pkg_alias2.pc
        - pkg_other_name.pc
        - second-mycomponent.pc
        - second.pc
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "pkg_other_name")
                self.cpp_info.set_property("pkg_config_aliases", ["pkg_alias1", "pkg_alias2"])
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "compo1")
                self.cpp_info.components["cmp1"].set_property("pkg_config_aliases", ["compo1_alias"])
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . first/0.3@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "first/0.3"

            def package_info(self):
                self.cpp_info.components["mycomponent"].requires.append("first::cmp1")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . second/0.2@")

    conanfile = textwrap.dedent("""
        [requires]
        second/0.2

        [generators]
        PkgConfigDeps
        """)
    client.save({"conanfile.txt": conanfile}, clean_first=True)
    client.run("install .")

    pc_content = client.load("compo1.pc")
    assert "Description: Conan component: pkg_other_name-compo1" in pc_content
    assert "Requires" not in pc_content

    pc_content = client.load("compo1_alias.pc")
    content = textwrap.dedent("""\
    Name: compo1_alias
    Description: Alias compo1_alias for compo1
    Version: 0.3
    Requires: compo1
    """)
    assert content == pc_content

    pc_content = client.load("pkg_other_name.pc")
    assert "Description: Conan package: pkg_other_name" in pc_content
    assert "Requires: compo1" in pc_content

    pc_content = client.load("pkg_alias1.pc")
    content = textwrap.dedent("""\
    Name: pkg_alias1
    Description: Alias pkg_alias1 for pkg_other_name
    Version: 0.3
    Requires: pkg_other_name
    """)
    assert content == pc_content

    pc_content = client.load("pkg_alias2.pc")
    content = textwrap.dedent("""\
    Name: pkg_alias2
    Description: Alias pkg_alias2 for pkg_other_name
    Version: 0.3
    Requires: pkg_other_name
    """)
    assert content == pc_content

    pc_content = client.load("second-mycomponent.pc")
    assert "Requires: compo1" == get_requires_from_content(pc_content)


def test_components_and_package_pc_creation_order():
    """
    Testing if the root package PC file name matches with any of the components one, the first one
    is not going to be created. Components have more priority than root package.

    Issue related: https://github.com/conan-io/conan/issues/10341
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "OpenCL")
                self.cpp_info.components["_opencl-headers"].set_property("pkg_config_name", "OpenCL")
                self.cpp_info.components["_opencl-other"].set_property("pkg_config_name", "OtherCL")
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . opencl/1.0@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            requires = "opencl/1.0"

            def package_info(self):
                self.cpp_info.components["comp"].set_property("pkg_config_name", "pkgb")
                self.cpp_info.components["comp"].requires.append("opencl::_opencl-headers")
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . pkgb/1.0@")

    conanfile = textwrap.dedent("""
        [requires]
        pkgb/1.0

        [generators]
        PkgConfigDeps
        """)
    client.save({"conanfile.txt": conanfile}, clean_first=True)
    client.run("install .")
    pc_files = [os.path.basename(i) for i in glob.glob(os.path.join(client.current_folder, '*.pc'))]
    pc_files.sort()
    # Let's check all the PC file names created just in case
    assert pc_files == ['OpenCL.pc', 'OtherCL.pc', 'pkgb.pc']
    pc_content = client.load("OpenCL.pc")
    assert "Name: OpenCL" in pc_content
    assert "Description: Conan component: OpenCL" in pc_content
    assert "Requires:" not in pc_content
    pc_content = client.load("pkgb.pc")
    assert "Requires: OpenCL" in get_requires_from_content(pc_content)


def test_pkgconfigdeps_with_test_requires():
    """
    PkgConfigDeps has to create any test requires declared on the recipe.

    Related issue: https://github.com/conan-io/conan/issues/11376
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.cpp_info.libs = ["lib%s"]
        """)
    with client.chdir("app"):
        client.save({"conanfile.py": conanfile % "app"})
        # client.run("create . --name=app --version=1.0")
        client.run("create . app/1.0@")
    with client.chdir("test"):
        client.save({"conanfile.py": conanfile % "test"})
        # client.run("create . --name=test --version=1.0")
        client.run("create . test/1.0@")
    # Create library having build and test requires
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class HelloLib(ConanFile):
            def build_requirements(self):
                self.test_requires('app/1.0')
                self.test_requires('test/1.0')
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -g PkgConfigDeps")
    assert "Description: Conan package: test" in client.load("test.pc")
    assert "Description: Conan package: app" in client.load("app.pc")


def test_with_editable_layout():
    """
    https://github.com/conan-io/conan/issues/11435
    """
    client = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save
        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def layout(self):
                self.cpp.source.includedirs = ["include"]
            def package_info(self):
                self.cpp_info.libs = ["mylib"]
        """)
    client.save({"dep/conanfile.py": dep,
                 "dep/include/header.h": "",
                 "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_requires("dep/0.1")})
    client.run("create dep")
    client.run("editable add dep dep/0.1")
    with client.chdir("pkg"):
        client.run("install . -g PkgConfigDeps")
        pc = client.load("dep.pc")
        assert "Libs: -lmylib" in pc
        assert 'includedir1=' in pc
        assert 'Cflags: -I"${includedir1}"' in pc


def test_tool_requires():
    """
    Testing if PC files are created for tool requires if build_context_activated/_suffix is used.

    Issue related: https://github.com/conan-io/conan/issues/11710
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . tool/1.0@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "libother")
                self.cpp_info.components["cmp1"].libs = ["other_cmp1"]
                self.cpp_info.components["cmp1"].set_property("pkg_config_name", "component1")
                self.cpp_info.components["cmp2"].libs = ["other_cmp2"]
                self.cpp_info.components["cmp3"].requires.append("cmp1")
                self.cpp_info.components["cmp3"].set_property("pkg_config_name", "component3")
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create . other/1.0@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import PkgConfigDeps

        class PkgConfigConan(ConanFile):
            name = "demo"
            version = "1.0"

            def build_requirements(self):
                self.build_requires("tool/1.0")
                self.build_requires("other/1.0")

            def generate(self):
                tc = PkgConfigDeps(self)
                tc.build_context_activated = ["other", "tool"]
                tc.build_context_suffix = {"tool": "_bt", "other": "_bo"}
                tc.generate()
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    pc_files = [os.path.basename(i) for i in glob.glob(os.path.join(client.current_folder, '*.pc'))]
    pc_files.sort()
    # Let's check all the PC file names created just in case
    assert pc_files == ['component1_bo.pc', 'component3_bo.pc',
                        'libother_bo-cmp2.pc', 'libother_bo.pc', 'tool_bt.pc']
    pc_content = client.load("tool_bt.pc")
    assert "Name: tool_bt" in pc_content
    pc_content = client.load("libother_bo.pc")
    assert "Name: libother_bo" in pc_content
    assert "Requires: component1_bo libother_bo-cmp2 component3_bo" == get_requires_from_content(pc_content)
    pc_content = client.load("component1_bo.pc")
    assert "Name: component1_bo" in pc_content
    pc_content = client.load("libother_bo-cmp2.pc")
    assert "Name: libother_bo-cmp2" in pc_content
    pc_content = client.load("component3_bo.pc")
    assert "Name: component3_bo" in pc_content
    assert "Requires: component1_bo" == get_requires_from_content(pc_content)


def test_tool_requires_not_created_if_no_activated():
    """
    Testing if there are no PC files created in no context are activated
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . tool/1.0@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):
            name = "demo"
            version = "1.0"
            generators = "PkgConfigDeps"

            def build_requirements(self):
                self.build_requires("tool/1.0")

        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default")
    pc_files = [os.path.basename(i) for i in glob.glob(os.path.join(client.current_folder, '*.pc'))]
    pc_files.sort()
    assert pc_files == []


def test_tool_requires_raise_exception_if_exist_both_require_and_build_one():
    """
    Testing if same dependency exists in both require and build require (without suffix)
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class PkgConfigConan(ConanFile):

            def package_info(self):
                self.cpp_info.libs = ["libtool"]
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . tool/1.0@")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import PkgConfigDeps

        class PkgConfigConan(ConanFile):
            name = "demo"
            version = "1.0"

            def requirements(self):
                self.requires("tool/1.0")

            def build_requirements(self):
                self.build_requires("tool/1.0")

            def generate(self):
                tc = PkgConfigDeps(self)
                tc.build_context_activated = ["tool"]
                tc.generate()
        """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -pr:h default -pr:b default", assert_error=True)
    assert "The packages ['tool'] exist both as 'require' and as 'build require'" in client.out



# coding=utf-8

import os
import unittest

import six
from mock import Mock, MagicMock

from conans import __version__
from conans.client.cache.cache import ClientCache
from conans.client.conf import get_default_client_conf, ConanClientConfigParser
from conans.client.rest.conan_requester import ConanRequester
from conans.client.tools import environment_append
from conans.client.tools.files import replace_in_file, save
from conans.errors import ConanException
from conans.paths import CACERT_FILE
from conans.test.utils.tools import temp_folder
from conans.test.utils.mocks import TestBufferConanOutput
from conans.util.files import normalize


class MockRequesterGet(Mock):
    verify = None

    def get(self, _, **kwargs):
        self.verify = kwargs.get('verify', None)


class ConanRequesterCacertPathTests(unittest.TestCase):

    @staticmethod
    def _create_requesters(cache_folder=None):
        cache = ClientCache(cache_folder or temp_folder(), TestBufferConanOutput())
        mock_requester = MockRequesterGet()
        requester = ConanRequester(cache.config, mock_requester)
        return requester, mock_requester, cache

    def test_default_no_verify(self):
        requester, mocked_requester, _ = self._create_requesters()
        requester.get(url="aaa", verify=False)
        self.assertEqual(mocked_requester.verify, False)

    def test_default_verify(self):
        requester, mocked_requester, cache = self._create_requesters()
        requester.get(url="aaa", verify=True)
        self.assertEqual(mocked_requester.verify, cache.config.cacert_path)

    def test_env_variable(self):
        file_path = os.path.join(temp_folder(), "whatever_cacert")
        save(file_path, "dummy content")
        with environment_append({"CONAN_CACERT_PATH": file_path}):
            requester, mocked_requester, _ = self._create_requesters()
            requester.get(url="aaa", verify=True)
            self.assertEqual(mocked_requester.verify, file_path)

    def test_cache_config(self):
        file_path = os.path.join(temp_folder(), "whatever_cacert")
        save(file_path, "")
        conan_conf = os.path.join(temp_folder(), "conan.conf")
        save(conan_conf, normalize(get_default_client_conf()))
        replace_in_file(conan_conf, "# cacert_path",
                        "cacert_path={}".format(file_path),
                        output=TestBufferConanOutput())
        config = ConanClientConfigParser(conan_conf)
        mocked_requester = MockRequesterGet()
        requester = ConanRequester(config, mocked_requester)
        requester.get(url="bbbb", verify=True)
        self.assertEqual(mocked_requester.verify, file_path)

    def test_non_existing_file(self):
        file_path = os.path.join(temp_folder(), "whatever_cacert")
        self.assertFalse(os.path.exists(file_path))
        with environment_append({"CONAN_CACERT_PATH": file_path}):
            with six.assertRaisesRegex(self, ConanException, "Configured file for 'cacert_path'"
                                                             " doesn't exist"):
                self._create_requesters()

    def test_non_existing_default_file(self):
        conan_user_home = temp_folder()
        default_cacert_path = os.path.join(conan_user_home, CACERT_FILE)
        self.assertFalse(os.path.exists(default_cacert_path))

        requester, mocked_requester, cache = self._create_requesters(conan_user_home)
        requester.get(url="aaa", verify=True)
        self.assertEqual(mocked_requester.verify, cache.config.cacert_path)
        self.assertEqual(cache.config.cacert_path, default_cacert_path)


class ConanRequesterHeadersTests(unittest.TestCase):
    def test_user_agent(self):
        cache_folder = temp_folder()
        cache = ClientCache(cache_folder, TestBufferConanOutput())
        mock_http_requester = MagicMock()
        requester = ConanRequester(cache.config, mock_http_requester)
        requester.get(url="aaa")
        headers = mock_http_requester.get.call_args[1]["headers"]
        self.assertIn("Conan/%s" % __version__, headers["User-Agent"])

        requester.get(url="aaa", headers={"User-Agent": "MyUserAgent"})
        headers = mock_http_requester.get.call_args[1]["headers"]
        self.assertEqual("MyUserAgent", headers["User-Agent"])



import unittest

from conans.test.utils.tools import GenConanfile, TestClient


class InstallParallelTest(unittest.TestCase):

    def test_basic_parallel_install(self):
        client = TestClient(default_server_user=True)
        threads = 1  # At the moment, not really parallel until output implements mutex
        counter = 4
        client.run("config set general.parallel_download=%s" % threads)
        client.save({"conanfile.py": GenConanfile()})

        for i in range(counter):
            client.run("create . pkg%s/0.1@user/testing" % i)
        client.run("upload * --all --confirm")
        client.run("remove * -f")

        # Lets consume the packages
        conanfile_txt = ["[requires]"]
        for i in range(counter):
            conanfile_txt.append("pkg%s/0.1@user/testing" % i)
        conanfile_txt = "\n".join(conanfile_txt)

        client.save({"conanfile.txt": conanfile_txt}, clean_first=True)
        client.run("install .")
        self.assertIn("Downloading binary packages in %s parallel threads" % threads, client.out)
        for i in range(counter):
            self.assertIn("pkg%s/0.1@user/testing: Package installed" % i, client.out)



import platform
import textwrap

from conans.client.tools import environment_append
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_profile_template():
    client = TestClient()
    tpl = textwrap.dedent("""
        [settings]
        os = {{ {"Darwin": "Macos"}.get(platform.system(), platform.system()) }}
        build_type = {{ os.getenv("MY_BUILD_TYPE") }}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1.jinja": tpl})
    with environment_append({"MY_BUILD_TYPE": "Debug"}):
        client.run("install . -pr=profile1.jinja")

    current_os = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "os={}".format(current_os)
    assert "build_type=Debug"


def test_profile_template_variables():
    client = TestClient()
    tpl = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        [settings]
        os = {{ a }}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1.jinja": tpl})
    client.run("install . -pr=profile1.jinja")
    assert "os=FreeBSD" in client.out


def test_profile_template_inclusion():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        {% import "profile_vars.jinja" as vars %}
        [settings]
        os = {{ vars.a }}
        """)
    tpl2 = textwrap.dedent("""
        {% set a = "FreeBSD" %}
        """)
    client.save({"conanfile.py": GenConanfile(),
                 "profile1.jinja": tpl1,
                 "profile_vars.jinja": tpl2})
    client.run("install . -pr=profile1.jinja")
    assert "os=FreeBSD" in client.out


def test_profile_template_profile_dir():
    client = TestClient()
    tpl1 = textwrap.dedent("""
        [conf]
        tools.toolchain:mydir = {{ os.path.join(profile_dir, "toolchain.cmake") }}
        """)
    conanfile = textwrap.dedent("""
        from conans import ConanFile, load
        class Pkg(ConanFile):
            def generate(self):
                content = load(self.conf["tools.toolchain:mydir"])
                self.output.info("CONTENT: {}".format(content))
        """)
    client.save({"conanfile.py": conanfile,
                 "anysubfolder/profile1.jinja": tpl1,
                 "anysubfolder/toolchain.cmake": "MyToolchainCMake!!!"})
    client.run("install . -pr=anysubfolder/profile1.jinja")
    assert "conanfile.py: CONTENT: MyToolchainCMake!!!" in client.out



import os
import time
from conans.client.installer import _PackageBuilder as _PackageBuilder_co, BinaryInstaller as BinaryInstaller_co
from sense2.client.conanfile.deb import run_deb_method
from conans.util.files import clean_dirty, is_dirty, make_read_only, mkdir, rmdir, save, set_dirty
from conans.client.generators import TXTGenerator
from conans.paths import BUILD_INFO, CONANINFO, RUN_LOG_NAME
from conans.client.packager import update_package_metadata
from conans.errors import (ConanException, ConanExceptionInUserConanfileMethod,
                           conanfile_exception_formatter, ConanInvalidConfiguration)
from conans.util.env_reader import get_env
from conans.util.tracer import log_package_built, log_package_got_from_local_cache
from conans.client.recorder.action_recorder import INSTALL_ERROR_BUILDING, INSTALL_ERROR_MISSING, \
    INSTALL_ERROR_MISSING_BUILD_FOLDER
from conans.client.source import retrieve_exports_sources


class _PackageBuilder(_PackageBuilder_co):
    
    def _deb(self, conanfile, pref, package_layout, conanfile_path):
        # FIXME: Is weak to assign here the recipe_hash
        manifest = package_layout.recipe_manifest()
        conanfile.info.recipe_hash = manifest.summary_hash

        # Creating ***info.txt files
        save(os.path.join(conanfile.folders.base_build, CONANINFO), conanfile.info.dumps())
        self._output.info("Generated %s" % CONANINFO)
        save(os.path.join(conanfile.folders.base_build, BUILD_INFO),
             TXTGenerator(conanfile).content)
        self._output.info("Generated %s" % BUILD_INFO)

        deb_id = pref.id
        # Do the actual copy, call the conanfile.package() method
        # While installing, the infos goes to build folder
        conanfile.folders.set_base_install(conanfile.folders.base_build)
        prev = run_deb_method(conanfile, deb_id, self._hook_manager, conanfile_path,
                                  pref.ref)

        update_package_metadata(prev, package_layout, deb_id, pref.ref.revision)

        if get_env("CONAN_READ_ONLY_CACHE", False):
            make_read_only(conanfile.folders.base_package)
        # FIXME: Conan 2.0 Clear the registry entry (package ref)
        return prev

    def build_package(self, node, keep_build, recorder, remotes):
        t1 = time.time()

        conanfile = node.conanfile
        pref = node.pref

        package_layout = self._cache.package_layout(pref.ref, conanfile.short_paths)
        base_source = package_layout.source()
        conanfile_path = package_layout.conanfile()
        base_package = package_layout.package(pref)
        base_deb = package_layout.deb(pref)
        
        base_build, skip_build = self._get_build_folder(conanfile, package_layout,
                                                               pref, keep_build, recorder)
        # PREPARE SOURCES
        if not skip_build:
            with package_layout.conanfile_write_lock(self._output):
                set_dirty(base_build)
                self._prepare_sources(conanfile, pref, package_layout, remotes)
                self._copy_sources(conanfile, base_source, base_build)

        # BUILD & PACKAGE
        with package_layout.conanfile_read_lock(self._output):
            self._output.info('Building your package in %s' % base_build)
            try:
                if getattr(conanfile, 'no_copy_source', False):
                    conanfile.folders.set_base_source(base_source)
                else:
                    conanfile.folders.set_base_source(base_build)

                conanfile.folders.set_base_build(base_build)
                conanfile.folders.set_base_imports(base_build)
                conanfile.folders.set_base_package(base_package)
                conanfile.folders.set_base_deb(base_deb)
                # In local cache, generators folder always in build_folder
                conanfile.folders.set_base_generators(base_build)

                if not skip_build:
                    # In local cache, install folder always is build_folder
                    conanfile.folders.set_base_install(base_build)
                    self._build(conanfile, pref)
                    clean_dirty(base_build)

                prev = self._package(conanfile, pref, package_layout, conanfile_path)
                assert prev
                prev1 = self._deb(conanfile, pref, package_layout, conanfile_path)
                assert prev1
                node.prev = prev
                log_file = os.path.join(base_build, RUN_LOG_NAME)
                log_file = log_file if os.path.exists(log_file) else None
                log_package_built(pref, time.time() - t1, log_file)
                recorder.package_built(pref)
            except ConanException as exc:
                recorder.package_install_error(pref, INSTALL_ERROR_BUILDING, str(exc),
                                               remote_name=None)
                raise exc

            return node.pref
        
        
class BinaryInstaller(BinaryInstaller_co):
    def __init__(self, app, recorder):
        super().__init__(app, recorder)
    
    def _build_package(self, node, output, keep_build, remotes):
        conanfile = node.conanfile
        # It is necessary to complete the sources of python requires, which might be used
        # Only the legacy python_requires allow this
        python_requires = getattr(conanfile, "python_requires", None)
        if python_requires and isinstance(python_requires, dict):  # Old legacy python_requires
            for python_require in python_requires.values():
                assert python_require.ref.revision is not None, \
                    "Installer should receive python_require.ref always"
                retrieve_exports_sources(self._remote_manager, self._cache,
                                         python_require.conanfile, python_require.ref, remotes)

        builder = _PackageBuilder(self._cache, output, self._hook_manager, self._remote_manager,
                                  self._generator_manager)
        pref = builder.build_package(node, keep_build, self._recorder, remotes)
        if node.graph_lock_node:
            node.graph_lock_node.prev = pref.revision
        return pref
    
    

def call_system_requirements(conanfile, output):
    try:
        return conanfile.system_requirements()
    except Exception as e:
        output.error("while executing system_requirements(): %s" % str(e))
        raise ConanException("Error in system requirements")


import textwrap

from conans.client.graph.graph import CONTEXT_BUILD, CONTEXT_HOST
from conans.model.profile import Profile
from conans.model.ref import ConanFileReference
from conans.test.integration.graph.core.cross_build._base_test_case import CrossBuildingBaseTestCase


class BuildRequiresInRecipeExample(CrossBuildingBaseTestCase):
    """ There is an application with a requirement 'lib', both of them build_requires
        a tool called 'breq' (build_machine) and this tool requires a 'breq_lib'.

        All these requirements are declared in the recipes
    """
    application = textwrap.dedent("""
        from conans import ConanFile

        class Application(ConanFile):
            name = "app"
            version = "testing"

            settings = "os"
            requires = "lib/testing@user/channel"

            def build_requirements(self):
                self.build_requires("breq/testing@user/channel")

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
    """)

    breq = CrossBuildingBaseTestCase.library_tpl.render(name="breq",
                                                        requires=["breq_lib/testing@user/channel", ])
    breq_lib = CrossBuildingBaseTestCase.library_tpl.render(name="breq_lib")
    lib = CrossBuildingBaseTestCase.library_tpl.render(
        name="lib", build_requires=[("breq/testing@user/channel", False), ])

    breq_lib_ref = ConanFileReference.loads("breq_lib/testing@user/channel")
    breq_ref = ConanFileReference.loads("breq/testing@user/channel")
    lib_ref = ConanFileReference.loads("lib/testing@user/channel")

    def setUp(self):
        super(BuildRequiresInRecipeExample, self).setUp()
        self._cache_recipe(self.breq_lib_ref, self.breq_lib)
        self._cache_recipe(self.breq_ref, self.breq)
        self._cache_recipe(self.lib_ref, self.lib)
        self._cache_recipe(self.app_ref, self.application)

    def test_crossbuilding(self):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        profile_build = Profile()
        profile_build.settings["os"] = "Build"
        profile_build.process_settings(self.cache)

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build)

        # Check HOST packages
        application = deps_graph.root.dependencies[0].dst
        self.assertEqual(len(application.dependencies), 2)
        self.assertEqual(application.conanfile.name, "app")
        self.assertEqual(application.context, CONTEXT_HOST)
        self.assertEqual(application.conanfile.settings.os, profile_host.settings['os'])

        lib_host = application.dependencies[0].dst
        self.assertEqual(lib_host.conanfile.name, "lib")
        self.assertEqual(lib_host.context, CONTEXT_HOST)
        self.assertEqual(lib_host.conanfile.settings.os, profile_host.settings['os'])

        # Check BUILD packages
        breq_application_build = application.dependencies[1].dst
        self.assertEqual(breq_application_build.conanfile.name, "breq")
        self.assertEqual(breq_application_build.context, CONTEXT_BUILD)
        self.assertEqual(str(breq_application_build.conanfile.settings.os), profile_build.settings['os'])

        breq_lib_build = lib_host.dependencies[0].dst
        self.assertNotEqual(breq_application_build, breq_lib_build)
        self.assertEqual(breq_lib_build.conanfile.name, "breq")
        self.assertEqual(breq_lib_build.context, CONTEXT_BUILD)
        self.assertEqual(str(breq_lib_build.conanfile.settings.os), profile_build.settings['os'])

        breq_lib_build = breq_application_build.dependencies[0].dst
        self.assertEqual(breq_lib_build.conanfile.name, "breq_lib")
        self.assertEqual(breq_lib_build.context, CONTEXT_BUILD)
        self.assertEqual(str(breq_lib_build.conanfile.settings.os), profile_build.settings['os'])



# coding=utf-8
import os
import textwrap
import unittest

import pytest
from parameterized import parameterized
from parameterized.parameterized import parameterized_class

from conans.model.ref import ConanFileReference
from conans.paths import DATA_YML
from conans.test.utils.scm import create_local_git_repo
from conans.test.utils.tools import TestClient
from conans.util.files import load


@pytest.mark.parametrize("scm_to_conandata", [True, False])
def test_shallow_none_string(scm_to_conandata):
    client = TestClient()
    client.run("config set general.scm_to_conandata={}".format('1' if scm_to_conandata else '0'))
    client.save({'conanfile.py': textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            scm = {"type": "git", "url": "https://github.com/repo/library.git",
                    "revision": "123456", "shallow": 'None' }
    """)})

    client.run('export . name/version@', assert_error=True)
    assert "ERROR: SCM value for 'shallow' must be of type 'bool' (found 'str')" in str(client.out)


@pytest.mark.tool_git
@parameterized_class([{"shallow": True}, {"shallow": False},
                      {"shallow": None},  # No value written in the recipe
                      {"shallow": 'None'}])  # Explicit 'None' written in the recipe
class GitShallowTestCase(unittest.TestCase):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conans.errors import ConanException
        from six import StringIO

        class Lib(ConanFile):
            scm = {{"type": "git", "url": "{url}", "revision": "{rev}", {shallow_attrib} }}

            def build(self):
                try:
                    mybuf = StringIO()
                    out = self.run("git describe --tags", output=mybuf)
                    self.output.info(">>> tags: {{}}".format(mybuf.getvalue()))
                except ConanException:
                    self.output.info(">>> describe-fails")
    """)

    ref = ConanFileReference.loads("name/version@user/channel")

    def _shallow_attrib_str(self):
        shallow_attrib_str = ""
        if self.shallow is not None:
            shallow_attrib_str = '"shallow": {}'.format(self.shallow)
        return shallow_attrib_str

    def _check_info_values(self, client):
        client.run("inspect {} -a scm".format(self.ref))  # Check we get a loadable conanfile.py
        if self.shallow in [None]:
            self.assertNotIn('shallow', str(client.out))
        elif self.shallow in [True]:  # This is the default value
            not_appears = 'shallow' not in str(client.out)
            value_explicit = 'shallow: True' in str(client.out)
            self.assertTrue(not_appears or value_explicit)
        elif self.shallow in ['None']:
            self.assertIn('shallow: None', str(client.out))
        else:
            self.assertIn('shallow: False', str(client.out))

    def test_export_scm_substituted(self):
        # Check the shallow value is substituted with the proper value
        client = TestClient()
        files = {'conanfile.py': self.conanfile.format(shallow_attrib=self._shallow_attrib_str(),
                                                       url='auto', rev='auto')}
        url, _ = create_local_git_repo(files=files)

        client.run_command('git clone "{}" .'.format(url))

        client.run("export . {}".format(self.ref))
        content = load(client.cache.package_layout(self.ref).conanfile())
        if self.shallow in [None, True]:
            self.assertNotIn("shallow", content)
        elif self.shallow in ['None']:
            self.assertIn('"shallow": None', content)
        else:
            self.assertIn('"shallow": False', content)

        self._check_info_values(client)

    def test_export_scm_to_conandata(self):
        # Check the shallow value is stored and propagated with the proper value
        client = TestClient()
        client.run("config set general.scm_to_conandata=1")
        files = {'conanfile.py': self.conanfile.format(shallow_attrib=self._shallow_attrib_str(),
                                                       url='auto', rev='auto')}
        url, _ = create_local_git_repo(files=files)
        client.run_command('git clone "{}" .'.format(url))

        client.run("export . {}".format(self.ref))
        content = load(os.path.join(client.cache.package_layout(self.ref).export(), DATA_YML))
        if self.shallow in [None, True]:
            self.assertNotIn('shallow', content)
        elif self.shallow in ['None']:
            self.assertIn('shallow: null', content)
        else:
            self.assertIn('shallow: false', content)

        self._check_info_values(client)

    @parameterized.expand([("c6cc15fa2f4b576bd", False), ("0.22.1", True)])
    def test_remote_build(self, revision, shallow_works):
        # Shallow works only with branches or tags
        client = TestClient()
        client.save({'conanfile.py':
                         self.conanfile.format(shallow_attrib=self._shallow_attrib_str(),
                                               url="https://github.com/conan-io/conan.git",
                                               rev=revision)})

        client.run("create . {}".format(self.ref))

        if self.shallow in [None, True] and shallow_works:
            self.assertIn(">>> describe-fails", client.out)
        else:
            self.assertIn(">>> tags: 0.22.1", client.out)



import os
import textwrap
import time
import unittest
from collections import Counter
from threading import Thread

from bottle import static_file, request
import pytest

from conans.client.downloaders.cached_file_downloader import CachedFileDownloader
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient, StoppableThreadBottle
from conans.util.env_reader import get_env
from conans.util.files import load, save, set_dirty


class DownloadCacheTest(unittest.TestCase):

    def test_download_skip(self):
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                exports = "*"
                def package(self):
                    self.copy("*")
            """)
        client.save({"conanfile.py": conanfile,
                     "header.h": "header"})
        client.run("create . mypkg/0.1@user/testing")
        client.run("upload * --all --confirm")
        cache_folder = temp_folder()
        log_trace_file = os.path.join(temp_folder(), "mylog.txt")
        client.run('config set storage.download_cache="%s"' % cache_folder)
        client.run('config set log.trace_file="%s"' % log_trace_file)
        client.run("remove * -f")
        client.run("install mypkg/0.1@user/testing")
        content = load(log_trace_file)
        self.assertEqual(6, content.count('"_action": "DOWNLOAD"'))
        # 6 files cached, plus "locks" folder = 7
        self.assertEqual(7, len(os.listdir(cache_folder)))

        os.remove(log_trace_file)
        client.run("remove * -f")
        client.run("install mypkg/0.1@user/testing")
        content = load(log_trace_file)
        self.assertEqual(0, content.count('"_action": "DOWNLOAD"'))
        self.assertIn("DOWNLOADED_RECIPE", content)
        self.assertIn("DOWNLOADED_PACKAGE", content)

        # removing the config downloads things
        client.run('config rm storage.download_cache')
        os.remove(log_trace_file)
        client.run("remove * -f")
        client.run('config set log.trace_file="%s"' % log_trace_file)
        client.run("install mypkg/0.1@user/testing")
        content = load(log_trace_file)
        # Not cached uses 7, because it downloads twice conaninfo.txt
        self.assertEqual(7, content.count('"_action": "DOWNLOAD"'))

        # restoring config cache works again
        os.remove(log_trace_file)
        client.run('config set storage.download_cache="%s"' % cache_folder)
        client.run("remove * -f")
        client.run("install mypkg/0.1@user/testing")
        content = load(log_trace_file)
        self.assertEqual(0, content.count('"_action": "DOWNLOAD"'))

    @pytest.mark.skipif(get_env("TESTING_REVISIONS_ENABLED", False), reason="No sense with revs")
    def test_corrupted_cache(self):
        # This test only works without revisions, because v1 has md5 file checksums, but v2 nop
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                exports = "*"
                def package(self):
                    self.copy("*")
            """)
        client.save({"conanfile.py": conanfile,
                     "header.h": "header"})
        client.run("create . mypkg/0.1@user/testing")
        client.run("upload * --all --confirm")
        cache_folder = temp_folder()
        client.run('config set storage.download_cache="%s"' % cache_folder)
        client.run("remove * -f")
        client.run("install mypkg/0.1@user/testing")
        for f in os.listdir(cache_folder):
            f = os.path.join(cache_folder, f)
            if not os.path.isfile(f):
                continue
            save(f, load(f) + "a")
        client.run("remove * -f")
        client.run("install mypkg/0.1@user/testing")
        self.assertIn("mypkg/0.1@user/testing: Downloaded package", client.out)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_dirty_download(self):
        # https://github.com/conan-io/conan/issues/8578
        client = TestClient(default_server_user=True)
        cache_folder = temp_folder()
        client.run('config set storage.download_cache="%s"' % cache_folder)
        client.save({"conanfile.py": GenConanfile().with_package_file("file.txt", "content")})
        client.run("create . pkg/0.1@")
        client.run("upload * --all -c")
        client.run("remove * -f")
        client.run("install pkg/0.1@")
        for f in os.listdir(cache_folder):
            # damage the file
            path = os.path.join(cache_folder, f)
            if os.path.isfile(path):
                save(path, "broken!")
                set_dirty(path)

        client.run("remove * -f")
        client.run("install pkg/0.1@")
        assert "pkg/0.1: Downloaded package" in client.out

    def test_user_downloads_cached(self):
        http_server = StoppableThreadBottle()

        file_path = os.path.join(temp_folder(), "myfile.txt")
        save(file_path, "some content")
        file_path_query = os.path.join(temp_folder(), "myfile2.txt")
        save(file_path_query, "some query")

        @http_server.server.get("/myfile.txt")
        def get_file():
            f = file_path_query if request.query else file_path
            return static_file(os.path.basename(f), os.path.dirname(f))

        http_server.run_server()

        client = TestClient()
        cache_folder = temp_folder()
        client.run('config set storage.download_cache="%s"' % cache_folder)
        # badchecksums are not cached
        conanfile = textwrap.dedent("""
           from conans import ConanFile, tools
           class Pkg(ConanFile):
               def source(self):
                   tools.download("http://localhost:%s/myfile.txt", "myfile.txt", md5="kk")
           """ % http_server.port)
        client.save({"conanfile.py": conanfile})
        client.run("source .", assert_error=True)
        self.assertIn("ConanException: md5 signature failed for", client.out)
        self.assertIn("Provided signature: kk", client.out)
        self.assertIn("Computed signature: 9893532233caff98cd083a116b013c0b", client.out)
        # There are 2 things in the cache, the "locks" folder and the .dirty file because failure
        self.assertEqual(2, len(os.listdir(cache_folder)))  # Nothing was cached

        # This is the right checksum
        conanfile = textwrap.dedent("""
            from conans import ConanFile, tools
            class Pkg(ConanFile):
                def source(self):
                    md5 = "9893532233caff98cd083a116b013c0b"
                    md5_2 = "0dc8a17658b1c7cfa23657780742a353"
                    tools.download("http://localhost:{0}/myfile.txt", "myfile.txt", md5=md5)
                    tools.download("http://localhost:{0}/myfile.txt?q=2", "myfile2.txt", md5=md5_2)
            """).format(http_server.port)
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        local_path = os.path.join(client.current_folder, "myfile.txt")
        self.assertTrue(os.path.exists(local_path))
        self.assertEqual("some content", client.load("myfile.txt"))
        local_path2 = os.path.join(client.current_folder, "myfile2.txt")
        self.assertTrue(os.path.exists(local_path2))
        self.assertEqual("some query", client.load("myfile2.txt"))

        # 2 files cached, plus "locks" folder = 3
        # "locks" folder + 2 files cached + .dirty file from previous failure
        self.assertEqual(4, len(os.listdir(cache_folder)))

        # remove remote file
        os.remove(file_path)
        os.remove(local_path)
        os.remove(local_path2)
        # Will use the cached one
        client.run("source .")
        self.assertTrue(os.path.exists(local_path))
        self.assertTrue(os.path.exists(local_path2))
        self.assertEqual("some content", client.load("myfile.txt"))
        self.assertEqual("some query", client.load("myfile2.txt"))

        # disabling cache will make it fail
        os.remove(local_path)
        os.remove(local_path2)
        client.run("config rm storage.download_cache")
        client.run("source .", assert_error=True)
        self.assertIn("ERROR: conanfile.py: Error in source() method, line 7", client.out)
        self.assertIn("Not found: http://localhost", client.out)

    def test_user_downloads_cached_newtools(self):
        http_server = StoppableThreadBottle()

        file_path = os.path.join(temp_folder(), "myfile.txt")
        save(file_path, "some content")
        file_path_query = os.path.join(temp_folder(), "myfile2.txt")
        save(file_path_query, "some query")

        @http_server.server.get("/myfile.txt")
        def get_file():
            f = file_path_query if request.query else file_path
            return static_file(os.path.basename(f), os.path.dirname(f))

        http_server.run_server()

        client = TestClient()
        cache_folder = temp_folder()
        save(client.cache.new_config_path, "tools.files.download:download_cache=%s" % cache_folder)
        # badchecksums are not cached
        conanfile = textwrap.dedent("""
           from conans import ConanFile
           from conan.tools.files import download
           class Pkg(ConanFile):
               def source(self):
                   download(self, "http://localhost:%s/myfile.txt", "myfile.txt", md5="kk")
           """ % http_server.port)
        client.save({"conanfile.py": conanfile})
        client.run("source .", assert_error=True)
        self.assertIn("ConanException: md5 signature failed for", client.out)
        self.assertIn("Provided signature: kk", client.out)
        self.assertIn("Computed signature: 9893532233caff98cd083a116b013c0b", client.out)
        # There are 2 things in the cache, the "locks" folder and the .dirty file because failure
        self.assertEqual(2, len(os.listdir(cache_folder)))  # Nothing was cached

        # This is the right checksum
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.files import download
            class Pkg(ConanFile):
                def source(self):
                    md5 = "9893532233caff98cd083a116b013c0b"
                    md5_2 = "0dc8a17658b1c7cfa23657780742a353"
                    download(self, "http://localhost:{0}/myfile.txt", "myfile.txt", md5=md5)
                    download(self, "http://localhost:{0}/myfile.txt?q=2", "myfile2.txt", md5=md5_2)
            """).format(http_server.port)
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        local_path = os.path.join(client.current_folder, "myfile.txt")
        self.assertTrue(os.path.exists(local_path))
        self.assertEqual("some content", client.load("myfile.txt"))
        local_path2 = os.path.join(client.current_folder, "myfile2.txt")
        self.assertTrue(os.path.exists(local_path2))
        self.assertEqual("some query", client.load("myfile2.txt"))

        # 2 files cached, plus "locks" folder = 3
        # "locks" folder + 2 files cached + .dirty file from previous failure
        self.assertEqual(4, len(os.listdir(cache_folder)))

        # remove remote file
        os.remove(file_path)
        os.remove(local_path)
        os.remove(local_path2)
        # Will use the cached one
        client.run("source .")
        self.assertTrue(os.path.exists(local_path))
        self.assertTrue(os.path.exists(local_path2))
        self.assertEqual("some content", client.load("myfile.txt"))
        self.assertEqual("some query", client.load("myfile2.txt"))

        # disabling cache will make it fail
        os.remove(local_path)
        os.remove(local_path2)
        save(client.cache.new_config_path, "")
        client.run("source .", assert_error=True)
        self.assertIn("ERROR: conanfile.py: Error in source() method, line 8", client.out)
        self.assertIn("Not found: http://localhost", client.out)

    @pytest.mark.skipif(get_env("TESTING_REVISIONS_ENABLED", False),
                        reason="Hybrid test with both v1 and v2")
    def test_revision0_v2_skip(self):
        client = TestClient(default_server_user=True)
        client.run("config set general.revisions_enabled=False")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                exports = "*"
                def package(self):
                    self.copy("*")
                def deploy(self):
                    self.copy("*")
            """)
        client.save({"conanfile.py": conanfile,
                     "header.h": "header"})
        client.run("create . mypkg/0.1@user/testing")
        client.run("upload * --all --confirm")

        client2 = TestClient(servers=client.servers)
        client2.run("config set general.revisions_enabled=True")
        cache_folder = temp_folder()
        client2.run('config set storage.download_cache="%s"' % cache_folder)
        client2.run("install mypkg/0.1@user/testing")
        self.assertEqual("header", client2.load("header.h"))

        # modify non-revisioned pkg
        client.save({"conanfile.py": conanfile,
                     "header.h": "header2"})
        client.run("create . mypkg/0.1@user/testing")
        client.run("upload * --all --confirm")

        client2.run("remove * -f")
        client2.run("install mypkg/0.1@user/testing")
        self.assertEqual("header2", client2.load("header.h"))


class CachedDownloaderUnitTest(unittest.TestCase):
    def setUp(self):
        cache_folder = temp_folder()

        class FakeFileDownloader(object):
            def __init__(self):
                self.calls = Counter()

            def download(self, url, file_path=None, *args, **kwargs):
                if "slow" in url:
                    time.sleep(0.5)
                self.calls[url] += 1
                if file_path:
                    save(file_path, url)
                else:
                    return url

        self.file_downloader = FakeFileDownloader()
        self.cached_downloader = CachedFileDownloader(cache_folder, self.file_downloader)

    def test_concurrent_locks(self):
        folder = temp_folder()

        def download(index):
            if index % 2:
                content = self.cached_downloader.download("slow_testurl")
                content = content.decode("utf-8")
            else:
                file_path = os.path.join(folder, "myfile%s.txt" % index)
                self.cached_downloader.download("slow_testurl", file_path)
                content = load(file_path)
            self.assertEqual(content, "slow_testurl")
            self.assertEqual(self.file_downloader.calls["slow_testurl"], 1)

        ps = []
        for i in range(8):
            thread = Thread(target=download, args=(i,))
            thread.start()
            ps.append(thread)

        for p in ps:
            p.join()

        self.assertEqual(self.file_downloader.calls["slow_testurl"], 1)

    def test_basic(self):
        folder = temp_folder()
        file_path = os.path.join(folder, "myfile.txt")
        self.cached_downloader.download("testurl", file_path)
        self.assertEqual(self.file_downloader.calls["testurl"], 1)
        self.assertEqual("testurl", load(file_path))

        # Try again, the count will be the same
        self.cached_downloader.download("testurl", file_path)
        self.assertEqual(self.file_downloader.calls["testurl"], 1)
        # Try direct content
        content = self.cached_downloader.download("testurl")
        self.assertEqual(content.decode("utf-8"), "testurl")
        self.assertEqual(self.file_downloader.calls["testurl"], 1)

        # Try another file
        file_path = os.path.join(folder, "myfile2.txt")
        self.cached_downloader.download("testurl2", file_path)
        self.assertEqual(self.file_downloader.calls["testurl2"], 1)
        self.assertEqual("testurl2", load(file_path))
        self.cached_downloader.download("testurl2", file_path)
        self.assertEqual(self.file_downloader.calls["testurl"], 1)
        self.assertEqual(self.file_downloader.calls["testurl2"], 1)

    def test_content_first(self):
        # first calling content without path also caches
        content = self.cached_downloader.download("testurl")
        self.assertEqual(content.decode("utf-8"), "testurl")  # content is binary here
        self.assertEqual(self.file_downloader.calls["testurl"], 1)
        # Now the file
        folder = temp_folder()
        file_path = os.path.join(folder, "myfile.txt")
        self.cached_downloader.download("testurl", file_path)
        self.assertEqual(self.file_downloader.calls["testurl"], 1)
        self.assertEqual("testurl", load(file_path))



import os
import platform
import re

import pytest

from conans.model.ref import ConanFileReference, PackageReference
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_msbuild_lib_template():
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=msbuild_lib")
    # Local flow works
    client.run("install . -if=install")
    client.run("build . -if=install")

    assert os.path.isfile(os.path.join(client.current_folder, "x64", "Release", "hello.lib"))
    client.run("export-pkg . hello/0.1@ -if=install")
    package_id = re.search(r"Packaging to (\S+)", str(client.out)).group(1)
    pref = PackageReference(ConanFileReference.loads("hello/0.1"), package_id)
    package_folder = client.cache.package_layout(pref.ref).package(pref)
    assert os.path.exists(os.path.join(package_folder, "include", "hello.h"))
    assert os.path.exists(os.path.join(package_folder, "lib", "hello.lib"))

    # Create works
    client.run("create .")
    assert "hello/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "hello/0.1: Hello World Debug!" in client.out

    # Create + shared works
    client.run("create . -o hello:shared=True")
    assert "hello/0.1: Hello World Release!" in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_msbuild_exe_template():
    client = TestClient(path_with_spaces=False)
    client.run("new greet/0.1 --template=msbuild_exe")
    # Local flow works
    client.run("install . -if=install")
    client.run("build . -if=install")

    # Create works
    client.run("create .")
    assert "greet/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "greet/0.1: Hello World Debug!" in client.out



import platform
import unittest
from parameterized import parameterized

import pytest

from conans.client.build.cmake import CMake
from conans.client.conf import get_default_settings_yml
from conans.model.settings import Settings
from conans.test.utils.mocks import MockSettings, ConanFileMock


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
class CMakeAppleTest(unittest.TestCase):
    @parameterized.expand([('x86', 'Macos', 'i386', 'MacOSX.platform'),
                           ('x86_64', 'Macos', 'x86_64', 'MacOSX.platform'),
                           ('armv7', 'Macos', 'armv7', 'MacOSX.platform'),
                           ('armv8', 'Macos', 'arm64', 'MacOSX.platform'),
                           ('x86', 'iOS', 'i386', 'iPhoneSimulator.platform'),
                           ('x86_64', 'iOS', 'x86_64', 'iPhoneSimulator.platform'),
                           ('armv7', 'iOS', 'armv7', 'iPhoneOS.platform'),
                           ('armv8', 'iOS', 'arm64', 'iPhoneOS.platform'),
                           ('x86', 'watchOS', 'i386', 'WatchSimulator.platform'),
                           ('x86_64', 'watchOS', 'x86_64', 'WatchSimulator.platform'),
                           ('armv7', 'watchOS', 'armv7', 'WatchOS.platform'),
                           ('armv8', 'watchOS', 'arm64', 'WatchOS.platform'),
                           ('x86', 'tvOS', 'i386', 'AppleTVSimulator.platform'),
                           ('x86_64', 'tvOS', 'x86_64', 'AppleTVSimulator.platform'),
                           ('armv7', 'tvOS', 'armv7', 'AppleTVOS.platform'),
                           ('armv8', 'tvOS', 'arm64', 'AppleTVOS.platform')
                           ])
    def test_cmake_definitions(self, conan_arch, conan_os, expected_arch, expected_os):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = conan_os
        settings.compiler = "apple-clang"
        settings.compiler.version = "11.0"
        settings.arch = conan_arch

        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)

        self.assertEqual(cmake.definitions["CMAKE_OSX_ARCHITECTURES"], expected_arch)
        self.assertIn(expected_os, cmake.definitions["CMAKE_OSX_SYSROOT"])

    @parameterized.expand([('iOS', 'iPhoneOS.platform'),
                           ('Macos', 'MacOSX.platform'),
                           ('watchOS', 'WatchOS.platform'),
                           ('tvOS', 'AppleTVOS.platform')
                           ])
    def test_build_folder_vars(self, conan_os, expected_os):
        settings = MockSettings({"os": conan_os,
                                 "compiler": "apple-clang",
                                 "compiler.version": "11.0",
                                 "arch": "ios_fat"})
        conanfile = ConanFileMock()
        conanfile.settings = settings
        cmake = CMake(conanfile)

        self.assertNotIn("CMAKE_OSX_ARCHITECTURES", cmake.definitions)
        self.assertIn(expected_os, cmake.definitions["CMAKE_OSX_SYSROOT"])



import textwrap

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class ConfigMethodTestCase(ConanV2ModeTestCase):

    def test_config_method(self):
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                def config(self):
                    pass
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@', assert_error=True)
        self.assertIn("Conan v2 incompatible: config() has been deprecated. Use config_options() and configure()",
                      t.out)



import os
import textwrap
from collections import OrderedDict

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID
from conans.test.utils.tools import TestClient, TestServer, GenConanfile
from conans.util.files import mkdir, rmdir, save


@pytest.fixture()
def client():
    c = TestClient(default_server_user=True)
    save(c.cache.settings_path, "os: [Windows, Macos, Linux, FreeBSD]\nos_build: [Windows, Macos]")
    save(c.cache.default_profile_path, "[settings]\nos=Windows")
    return c


def test_not_found_package_dirty_cache(client):
    # Conan does a lock on the cache, and even if the package doesn't exist
    # left a trailing folder with the filelocks. This test checks
    # it will be cleared
    client.save({"conanfile.py": GenConanfile("Hello", "0.1")})
    client.run("create . lasote/testing")
    client.run("upload * --all --confirm")
    client.run('remove "*" -f')
    client.run(" install hello/0.1@lasote/testing", assert_error=True)
    assert "Unable to find 'hello/0.1@lasote/testing'" in client.out
    # This used to fail in Windows, because of the trailing lock
    client.run("remove * -f")
    client.run(" install Hello/0.1@lasote/testing")


def test_install_reference_txt(client):
    # Test to check the "conan install <path> <reference>" command argument
    client.save({"conanfile.txt": ""})
    client.run("info .")
    assert "conanfile.txt" in str(client.out).splitlines()


def test_install_reference_error(client):
    # Test to check the "conan install <path> <reference>" command argument
    client.run("install Pkg/0.1@myuser/testing user/testing", assert_error=True)
    assert "ERROR: A full reference was provided as first argument" in client.out


def test_install_reference(client):
    # Test to check the "conan install <path> <reference>" command argument
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            def build(self):
                self.output.info("REF: %s, %s, %s, %s"
                                 % (self.name, self.version, self.user, self.channel))
        """)
    client.save({"conanfile.py": conanfile})
    client.run(" install . Pkg/0.1@myuser/testing")
    client.run(" info .")
    assert "Pkg/0.1@myuser/testing" in client.out
    client.run("build .")
    assert "REF: Pkg, 0.1, myuser, testing" in client.out

    # Trying with partial name
    conanfile = conanfile + "    name = 'Other'\n"
    client.save({"conanfile.py": conanfile})
    # passing the wrong package name raises
    client.run(" install . Pkg/0.1@myuser/testing", assert_error=True)
    assert "ERROR: Package recipe with name Pkg!=Other" in client.out
    # Partial reference works
    client.run(" install . 0.1@myuser/testing")
    client.run("build .")
    assert "REF: Other, 0.1, myuser, testing" in client.out
    # And also full reference matching
    client.run(" install . Other/0.1@myuser/testing")
    client.run("build .")
    assert "REF: Other, 0.1, myuser, testing" in client.out

    # Trying with partial name and version
    conanfile = conanfile + "    version = '0.2'\n"
    client.save({"conanfile.py": conanfile})
    # passing the wrong package name raises
    client.run(" install . Other/0.1@myuser/testing", assert_error=True)
    assert "ERROR: Package recipe with version 0.1!=0.2" in client.out
    # Partial reference works
    client.run(" install . myuser/testing")
    client.run("build .")
    assert "REF: Other, 0.2, myuser, testing" in client.out
    # And also full reference matching
    client.run(" install . Other/0.2@myuser/testing")
    client.run("build .")
    assert "REF: Other, 0.2, myuser, testing" in client.out


def test_four_subfolder_install(client):
    # https://github.com/conan-io/conan/issues/3950
    client.save({"path/to/sub/folder/conanfile.txt": ""})
    # If this doesn't, fail, all good
    client.run(" install path/to/sub/folder")


def test_install_system_requirements(client):
    client.save({"conanfile.py": textwrap.dedent("""
        from conans import ConanFile
        class MyPkg(ConanFile):
            def system_requirements(self):
                self.output.info("Running system requirements!!")
        """)})
    client.run(" install .")
    assert "Running system requirements!!" in client.out
    client.run("export . Pkg/0.1@lasote/testing")
    client.run(" install Pkg/0.1@lasote/testing --build")
    assert "Running system requirements!!" in client.out
    client.run("upload * --all --confirm")
    client.run('remove "*" -f')
    client.run(" install Pkg/0.1@lasote/testing")
    assert "Running system requirements!!" in client.out


def test_install_transitive_pattern(client):
    # Make sure a simple conan install doesn't fire package_info() so self.package_folder breaks
    client.save({"conanfile.py": textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            options = {"shared": [True, False, "header"]}
            default_options = "shared=False"
            def package_info(self):
                self.output.info("PKG OPTION: %s" % self.options.shared)
        """)})
    client.run("create . Pkg/0.1@user/testing -o shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    client.save({"conanfile.py": textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            requires = "Pkg/0.1@user/testing"
            options = {"shared": [True, False, "header"]}
            default_options = "shared=False"
            def package_info(self):
                self.output.info("PKG2 OPTION: %s" % self.options.shared)
        """)})

    client.run("create . Pkg2/0.1@user/testing -o *:shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    client.run(" install Pkg2/0.1@user/testing -o *:shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    # Priority of non-scoped options
    client.run("create . Pkg2/0.1@user/testing -o shared=header -o *:shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install Pkg2/0.1@user/testing -o shared=header -o *:shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence of exact named option
    client.run("create . Pkg2/0.1@user/testing -o *:shared=True -o Pkg2:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install Pkg2/0.1@user/testing -o *:shared=True -o Pkg2:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence of exact named option reverse
    client.run("create . Pkg2/0.1@user/testing -o *:shared=True -o Pkg:shared=header "
               "--build=missing")
    assert "Pkg/0.1@user/testing: Calling build()" in client.out
    assert "Pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    client.run(" install Pkg2/0.1@user/testing -o *:shared=True -o Pkg:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: True" in client.out
    # Prevalence of alphabetical pattern
    client.run("create . Pkg2/0.1@user/testing -o *:shared=True -o Pkg2*:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install Pkg2/0.1@user/testing -o *:shared=True -o Pkg2*:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence of alphabetical pattern, opposite order
    client.run("create . Pkg2/0.1@user/testing -o Pkg2*:shared=header -o *:shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install Pkg2/0.1@user/testing -o Pkg2*:shared=header -o *:shared=True")
    assert "Pkg/0.1@user/testing: PKG OPTION: True" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    # Prevalence and override of alphabetical pattern
    client.run("create . Pkg2/0.1@user/testing -o *:shared=True -o Pkg*:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out
    client.run(" install Pkg2/0.1@user/testing -o *:shared=True -o Pkg*:shared=header")
    assert "Pkg/0.1@user/testing: PKG OPTION: header" in client.out
    assert "Pkg2/0.1@user/testing: PKG2 OPTION: header" in client.out


def test_install_package_folder(client):
    # Make sure a simple conan install doesn't fire package_info() so self.package_folder breaks
    client.save({"conanfile.py": textwrap.dedent("""\
        from conans import ConanFile
        import os
        class Pkg(ConanFile):
            def package_info(self):
                self.dummy_doesnt_exist_not_break
                self.output.info("Hello")
                self.env_info.PATH = os.path.join(self.package_folder, "bin")
        """)})
    client.run(" install .")
    assert "Hello" not in client.out
    assert "conanfile.py: Generated conaninfo.txt" in client.out


def test_install_cwd(client):
    client.save({"conanfile.py": GenConanfile("Hello", "0.1").with_setting("os")})
    client.run("export . lasote/stable")
    client.save({"conanfile.txt": "[requires]\nHello/0.1@lasote/stable"}, clean_first=True)

    client.run("install . --build=missing -s os_build=Windows --install-folder=win_dir")
    assert "Hello/0.1@lasote/stable from local cache" in client.out
    client.run("install . --build=missing -s os=Macos -s os_build=Macos "
               "--install-folder=os_dir")
    conaninfo = client.load("win_dir/conaninfo.txt")
    assert "os=Windows" in conaninfo
    assert "os=Macos" not in conaninfo
    conaninfo = client.load("os_dir/conaninfo.txt")
    assert "os=Windows" not in conaninfo
    assert "os=Macos" in conaninfo


def test_install_reference_not_conanbuildinfo(client):
    client.save({"conanfile.py": GenConanfile("Hello", "0.1").with_setting("os")})
    client.run("create . conan/stable")
    client.save({}, clean_first=True)
    client.run("install Hello/0.1@conan/stable")
    assert not os.path.exists(os.path.join(client.current_folder, "conanbuildinfo.txt"))


def test_install_with_profile(client):
    # Test for https://github.com/conan-io/conan/pull/2043
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            settings = "os"
            def requirements(self):
                self.output.info("PKGOS=%s" % self.settings.os)
        """)

    client.save({"conanfile.py": conanfile})
    client.run("profile new myprofile")
    client.run("profile update settings.os=Linux myprofile")
    client.run("install . -pr=myprofile --build")
    assert "PKGOS=Linux" in client.out
    mkdir(os.path.join(client.current_folder, "myprofile"))
    client.run("install . -pr=myprofile")
    client.run("profile new myotherprofile")
    client.run("profile update settings.os=FreeBSD myotherprofile")
    client.run("install . -pr=myotherprofile")
    assert "PKGOS=FreeBSD" in client.out
    client.save({"myotherprofile": "Some garbage without sense [garbage]"})
    client.run("install . -pr=myotherprofile")
    assert "PKGOS=FreeBSD" in client.out
    client.run("install . -pr=./myotherprofile", assert_error=True)
    assert "Error while parsing line 0" in client.out


def test_install_with_path_errors(client):
    # Install without path param not allowed
    client.run("install", assert_error=True)
    assert "ERROR: Exiting with code: 2" in client.out

    # Path with wrong conanfile.txt path
    client.run("install not_real_dir/conanfile.txt --install-folder subdir", assert_error=True)
    assert "Conanfile not found" in client.out

    # Path with wrong conanfile.py path
    client.run("install not_real_dir/conanfile.py --install-folder build", assert_error=True)
    assert "Conanfile not found" in client.out


def test_install_broken_reference(client):
    client.save({"conanfile.py": GenConanfile()})
    client.run("export . Hello/0.1@lasote/stable")
    client.run("remote add_ref Hello/0.1@lasote/stable default")
    ref = ConanFileReference.loads("Hello/0.1@lasote/stable")
    # Because the folder is removed, the metadata is removed and the
    # origin remote is lost
    rmdir(os.path.join(client.cache.package_layout(ref).base_folder()))
    client.run("install Hello/0.1@lasote/stable", assert_error=True)
    assert "ERROR: Unable to find 'Hello/0.1@lasote/stable' in remotes" in client.out

    # If it was associated, it has to be desasociated
    client.run("remote remove_ref Hello/0.1@lasote/stable")
    client.run("install Hello/0.1@lasote/stable", assert_error=True)
    assert "ERROR: Unable to find 'Hello/0.1@lasote/stable' in remotes" in client.out


def test_install_argument_order(client):
    # https://github.com/conan-io/conan/issues/2520
    conanfile_boost = textwrap.dedent("""
        from conans import ConanFile
        class BoostConan(ConanFile):
            name = "boost"
            version = "0.1"
            options = {"shared": [True, False]}
            default_options = "shared=True"
        """)
    conanfile = GenConanfile().with_require("boost/0.1@conan/stable")

    client.save({"conanfile.py": conanfile,
                 "conanfile_boost.py": conanfile_boost})
    client.run("create conanfile_boost.py conan/stable")
    client.run("install . -o boost:shared=True --build=missing")
    output_0 = "%s" % client.out
    client.run("install . -o boost:shared=True --build missing")
    output_1 = "%s" % client.out
    client.run("install -o boost:shared=True . --build missing")
    output_2 = "%s" % client.out
    client.run("install -o boost:shared=True --build missing .")
    output_3 = "%s" % client.out
    assert "ERROR" not in output_3
    assert output_0 == output_1
    assert output_1 == output_2
    assert output_2 == output_3

    client.run("install -o boost:shared=True --build boost . --build missing")
    output_4 = "%s" % client.out
    client.run("install -o boost:shared=True --build missing --build boost .")
    output_5 = "%s" % client.out
    assert output_4 == output_5


def test_install_anonymous(client):
    # https://github.com/conan-io/conan/issues/4871
    client.save({"conanfile.py": GenConanfile("Pkg", "0.1")})
    client.run("create . lasote/testing")
    client.run("upload * --confirm --all")

    client2 = TestClient(servers=client.servers, users={})
    client2.run("install Pkg/0.1@lasote/testing")
    assert "Pkg/0.1@lasote/testing: Package installed" in client2.out


def test_install_without_ref(client):
    client.save({"conanfile.py": GenConanfile("lib", "1.0")})
    client.run('create .')
    assert "lib/1.0: Package '{}' created".format(NO_SETTINGS_PACKAGE_ID) in client.out

    client.run('upload lib/1.0 -c --all')
    assert "Uploaded conan recipe 'lib/1.0' to 'default'" in client.out

    client.run('remove "*" -f')

    # This fails, Conan thinks this is a path
    client.run('install lib/1.0', assert_error=True)
    fake_path = os.path.join(client.current_folder, "lib", "1.0")
    assert "Conanfile not found at {}".format(fake_path) in client.out

    # Try this syntax to upload too
    client.run('install lib/1.0@')
    client.run('upload lib/1.0@ -c --all')


def test_install_disabled_remote(client):
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload * --confirm --all -r default")
    client.run("remote disable default")
    client.run("install Pkg/0.1@lasote/testing -r default")
    assert "Pkg/0.1@lasote/testing: Already installed!" in client.out
    client.run("remote enable default")
    client.run("install Pkg/0.1@lasote/testing -r default")
    client.run("remote disable default")
    client.run("install Pkg/0.1@lasote/testing --update", assert_error=True)
    assert "ERROR: Remote 'default' is disabled" in client.out


def test_install_skip_disabled_remote():
    client = TestClient(servers=OrderedDict({"default": TestServer(),
                                             "server2": TestServer(),
                                             "server3": TestServer()}),
                        users={"default": [("lasote", "mypass")],
                               "server3": [("lasote", "mypass")]})
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload * --confirm --all -r default")
    client.run("upload * --confirm --all -r server3")
    client.run("remove * -f")
    client.run("remote disable default")
    client.run("install Pkg/0.1@lasote/testing", assert_error=False)
    assert "Trying with 'default'..." not in client.out


def test_install_without_update_fail(client):
    # https://github.com/conan-io/conan/issues/9183
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . zlib/1.0@")
    client.run("upload * --confirm --all -r default")
    client.save({"conanfile.py": GenConanfile().with_requires("zlib/1.0")})
    client.run("remote disable default")
    client.run("install .")
    assert "zlib/1.0: Already installed" in client.out


def test_install_version_range_reference(client):
    # https://github.com/conan-io/conan/issues/5905
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . pkg/0.1@user/channel")
    client.run("install pkg/[*]@user/channel")
    assert "pkg/0.1@user/channel from local cache - Cache" in client.out
    client.run("install pkg/[0.*]@user/channel")
    assert "pkg/0.1@user/channel from local cache - Cache" in client.out


def test_install_error_never(client):
    client.save({"conanfile.py": GenConanfile("Hello0", "0.1")})
    client.run("create .")
    client.run("install . --build never --build missing", assert_error=True)
    assert "ERROR: --build=never not compatible with other options" in client.out
    client.run("install conanfile.py --build never --build Hello", assert_error=True)
    assert "ERROR: --build=never not compatible with other options" in client.out
    client.run("install ./conanfile.py --build never --build outdated", assert_error=True)
    assert "ERROR: --build=never not compatible with other options" in client.out


class TestCliOverride:

    def test_install_cli_override(self, client):
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . zlib/1.0@")
        client.run("create . zlib/2.0@")
        client.save({"conanfile.py": GenConanfile().with_requires("zlib/1.0")})
        client.run("install . --require-override=zlib/2.0")
        assert "zlib/2.0: Already installed" in client.out

    def test_install_cli_override_in_conanfile_txt(self, client):
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . zlib/1.0@")
        client.run("create . zlib/2.0@")
        client.save({"conanfile.txt": textwrap.dedent("""\
        [requires]
        zlib/1.0
        """)}, clean_first=True)
        client.run("install . --require-override=zlib/2.0")
        assert "zlib/2.0: Already installed" in client.out

    def test_install_ref_cli_override(self, client):
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . zlib/1.0@")
        client.run("create . zlib/1.1@")
        client.save({"conanfile.py": GenConanfile().with_requires("zlib/1.0")})
        client.run("create . pkg/1.0@")
        client.run("install pkg/1.0@ --require-override=zlib/1.1")
        assert "zlib/1.1: Already installed" in client.out

    def test_create_cli_override(self, client):
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . zlib/1.0@")
        client.run("create . zlib/2.0@")
        client.save({"conanfile.py": GenConanfile().with_requires("zlib/1.0"),
                     "test_package/conanfile.py": GenConanfile().with_test("pass")})
        client.run("create . pkg/0.1@ --require-override=zlib/2.0")
        assert "zlib/2.0: Already installed" in client.out

    def test_conditional(self, client):
        # https://github.com/conan-io/conan/issues/10045
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . cuda/10.2@")
        client.run("create . cuda/10.3@")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                options = {"with_cuda":[True,False]}
                default_options = {"with_cuda":True}
                def requirements(self):
                    if self.options.with_cuda:
                        self.requires('cuda/[>=10.1 <=11.4.2]')
            """)
        client.save({"conanfile.py": conanfile})
        client.run("install .")  # Check that without override, it resolves to 10.3
        assert "cuda/10.3" in client.out
        assert "cuda/10.2" not in client.out
        # Now check overrides for different commands
        client.run("create . opencv/1.0@ --require-override=cuda/10.2")
        assert "cuda/10.2" in client.out
        assert "cuda/10.3" not in client.out
        client.run("install . --require-override=cuda/10.2")
        assert "cuda/10.2" in client.out
        assert "cuda/10.3" not in client.out
        client.run("install opencv/1.0@ --require-override=cuda/10.2")
        assert "cuda/10.2" in client.out
        assert "cuda/10.3" not in client.out


def test_install_bintray_warning():
    server = TestServer(complete_urls=True)
    from conans.client.graph import proxy
    proxy.DEPRECATED_CONAN_CENTER_BINTRAY_URL = server.fake_url  # Mocking!
    client = TestClient(servers={"conan-center": server},
                        users={"conan-center": [("lasote", "mypass")]})
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . zlib/1.0@lasote/testing")
    client.run("upload zlib/1.0@lasote/testing --all -r conan-center")
    client.run("remove * -f")
    client.run("install zlib/1.0@lasote/testing -r conan-center")
    assert "WARN: Remote https://conan.bintray.com is deprecated and will be shut down " \
           "soon" in client.out
    client.run("install zlib/1.0@lasote/testing -r conan-center -s build_type=Debug")
    assert "WARN: Remote https://conan.bintray.com is deprecated and will be shut down " \
           "soon" not in client.out


def test_package_folder_available_consumer():
    """
    The package folder is not available when doing a consumer conan install "."
    We don't want to provide the package folder for the "cmake install" nor the "make install",
    as a consumer you could call the build system and pass the prefix PATH manually.
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
    from conans import ConanFile
    from conan.tools.cmake import cmake_layout
    class HelloConan(ConanFile):

        settings = "os", "arch", "build_type"

        def layout(self):
            cmake_layout(self)

        def generate(self):
            self.output.warn("Package folder is None? {}".format(self.package_folder is None))
    """)
    client.save({"conanfile.py": conanfile})

    # Installing it with "install ." with output folder
    client.run("install . -of=my_build")
    assert "WARN: Package folder is None? True" in client.out

    # Installing it with "install ." without output folder
    client.run("install .")
    assert "WARN: Package folder is None? True" in client.out



import platform
import unittest

from mock import Mock

from conans.client.generators.scons import SConsGenerator
from conans.model.build_info import CppInfo, DepCppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings


class SConsGeneratorTest(unittest.TestCase):

    def test_variables_setup(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "")
        cpp_info.defines = ["MYDEFINE1"]
        cpp_info.version = "0.1"
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        ref = ConanFileReference.loads("MyPkg2/3.2.3@lasote/stables")
        cpp_info = CppInfo(ref.name, "")
        cpp_info.defines = ["MYDEFINE2"]
        cpp_info.version = "3.2.3"
        conanfile.deps_cpp_info.add(ref.name, DepCppInfo(cpp_info))
        generator = SConsGenerator(conanfile)
        content = generator.content
        scons_lines = content.splitlines()
        self.assertIn("        \"CPPDEFINES\"  : ['MYDEFINE2', 'MYDEFINE1'],", scons_lines)
        self.assertIn("        \"CPPDEFINES\"  : ['MYDEFINE1'],", scons_lines)
        self.assertIn("        \"CPPDEFINES\"  : ['MYDEFINE2'],", scons_lines)
        self.assertIn('    "conan_version" : "None",', scons_lines)
        self.assertIn('    "MyPkg_version" : "0.1",', scons_lines)
        self.assertIn('    "MyPkg2_version" : "3.2.3",', scons_lines)

    def test_system_frameworks_libs(self):
        # https://github.com/conan-io/conan/issues/7301
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        cpp_info = CppInfo("MyPkg", "/rootpath")
        cpp_info.version = "0.1"
        cpp_info.libs = ["mypkg"]
        cpp_info.system_libs = ["pthread"]
        cpp_info.frameworks = ["cocoa"]
        cpp_info.frameworkdirs = ["frameworks"]
        cpp_info.filter_empty = False
        conanfile.deps_cpp_info.add("MyPkg", DepCppInfo(cpp_info))
        generator = SConsGenerator(conanfile)
        content = generator.content
        scons_lines = content.splitlines()
        self.assertIn('        "LIBS"        : [\'mypkg\', \'pthread\'],', scons_lines)
        self.assertIn('        "FRAMEWORKS"  : [\'cocoa\'],', scons_lines)
        if platform.system() == "Windows":
            self.assertIn('        "FRAMEWORKPATH"  : [\'/rootpath\\\\frameworks\'],', scons_lines)
        else:
            self.assertIn('        "FRAMEWORKPATH"  : [\'/rootpath/frameworks\'],', scons_lines)
        self.assertIn('    "MyPkg_version" : "0.1",', scons_lines)



import textwrap

from conans.test.utils.tools import TestClient
from conans import __version__


def test_conan_version():

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan import conan_version
        from conan.tools.scm import Version
        from conans import __version__

        class pkg(ConanFile):

            def generate(self):
                assert __version__ == str(conan_version)
                assert isinstance(conan_version, Version)
                print(f"current version: {conan_version}")
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("install .")
    assert f"current version: {__version__}" in client.out



import textwrap

from jinja2 import Template

_makefile_am = """
{% if main %}
bin_PROGRAMS = {{ main }}
{{main}}_SOURCES = {{ main_srcs }}
{% endif %}
{% if lib %}
lib_LIBRARIES = {{ lib }}
{{lib.replace(".", "_")}}_SOURCES = {{ lib_srcs }}
{% endif %}
{% if main and lib %}
{{main}}_LDADD = {{ lib }}
{% endif %}

"""
# newline at the end is important: m4: INTERNAL ERROR: recursive push_string!


def gen_makefile_am(**context):
    t = Template(_makefile_am)
    return t.render(**context)


_configure_ac = """
AC_INIT([main], [1.0], [some@email.com])
AM_INIT_AUTOMAKE([-Wall -Werror foreign])
AC_PROG_CXX
AC_PROG_RANLIB
AM_PROG_AR
AC_CONFIG_FILES([Makefile])
AC_OUTPUT

"""
# newline at the end is important: m4: INTERNAL ERROR: recursive push_string!


def gen_configure_ac(**context):
    t = Template(_configure_ac)
    return t.render(**context)


def gen_makefile(**context):
    makefile = textwrap.dedent("""\
        .PHONY: all
        all: apps libs

        apps: {% for s in apps %} {{s}} {% endfor %}

        libs: {% for s in libs %} lib{{s}}.a {% endfor %}

        {%- set link_libs = namespace(str='') %}
        {% for lib in libs %}
        {%- set link_libs.str = link_libs.str + ' -l' + lib|string  %}
        lib{{lib}}.a: {{lib}}.o
        	$(AR) rcs lib{{lib}}.a {{lib}}.o {%if static_runtime%}--static{%endif%}
        {% endfor %}

        {% for s in apps %}
        {{s}}: {{s}}.o libs
        	$(CXX) $(CXXFLAGS) $(CPPFLAGS) $(LDFLAGS) -o {{s}} {{s}}.o $(LIBS) {{link_libs.str}} -L. {%if static_runtime%}--static{%endif%}
        {% endfor %}
        """)

    t = Template(makefile)
    return t.render(**context)



import os
import platform
from shutil import copy

import mock
import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.genconanfile import GenConanfile
from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.scm import create_local_git_repo
from conans.test.utils.tools import TestClient, zipdir

app_name = "Release/my_app.exe" if platform.system() == "Windows" else "my_app"


@pytest.mark.parametrize("no_copy_source", ["False", "True"])
def test_exports_source_with_src_subfolder(no_copy_source):
    """If we have the sources in a subfolder, specifying it in the self.folders.source will
    work both locally (conan build) or in the cache (exporting the sources)"""
    conan_file = GenConanfile() \
        .with_name("app").with_version("1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_exports_sources("my_src/*")\
        .with_cmake_build()\
        .with_class_attribute("no_copy_source={}".format(no_copy_source))

    conan_file = str(conan_file)
    conan_file += """
    def layout(self):
        self.folders.source = "my_src"
        self.folders.build = str(self.settings.build_type)
    """
    cmake = gen_cmakelists(appname="my_app", appsources=["main.cpp"])
    app = gen_function_cpp(name="main")

    client = TestClient()
    client.save({"conanfile.py": conan_file,
                 "my_src/main.cpp": app,
                 "my_src/CMakeLists.txt": cmake})
    client.run("install . -if=install")
    client.run("build . -if=install")
    assert os.path.exists(os.path.join(client.current_folder, "Release", app_name))
    client.run("create . ")
    assert "Created package revision" in client.out


def test_exports():
    """If we have some sources in the root (like the CMakeLists.txt)
    we don't declare folders.source"""
    conan_file = GenConanfile() \
        .with_name("app").with_version("1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_exports("*.py") \
        .with_import("from my_tools import FOO")

    conan_file = str(conan_file)
    conan_file += """
    def layout(self):
        self.folders.source = "my_src"
    def build(self):
        # This FOO comes from the my_tools.py
        self.output.warn("FOO: {}".format(FOO))
    """

    client = TestClient()
    client.save({"conanfile.py": conan_file,
                 "my_tools.py": "FOO=1"})
    client.run("install . -if=install")
    client.run("build . -if=install")
    assert "FOO: 1" in client.out

    client.run("create . ")
    assert "FOO: 1" in client.out


def test_exports_source_without_subfolder():
    """If we have some sources in the root (like the CMakeLists.txt)
    we don't declare folders.source"""
    conan_file = GenConanfile() \
        .with_name("app").with_version("1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_exports_sources("CMakeLists.txt", "my_src/*")\
        .with_cmake_build()

    conan_file = str(conan_file)
    conan_file += """
    def layout(self):
        self.folders.build = str(self.settings.build_type)
    """
    cmake = gen_cmakelists(appname="my_app", appsources=["my_src/main.cpp"])
    app = gen_function_cpp(name="main")

    client = TestClient()
    client.save({"conanfile.py": conan_file,
                 "my_src/main.cpp": app,
                 "CMakeLists.txt": cmake})
    client.run("install . -if=install")
    client.run("build . -if=install")
    assert os.path.exists(os.path.join(client.current_folder, "Release", app_name))
    client.run("create . ")
    assert "Created package revision" in client.out


def test_scm_with_source_layout():
    """If we have the sources in git repository"""
    conan_file = GenConanfile() \
        .with_name("app").with_version("1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_scm({"type": "git", "revision": "auto", "url": "auto"})\
        .with_cmake_build()

    conan_file = str(conan_file)
    conan_file += """
    def layout(self):
        self.folders.source = "my_src"
        self.folders.build = "build_{}".format(self.settings.build_type)
    """
    cmake = gen_cmakelists(appname="my_app", appsources=["main.cpp"])
    app = gen_function_cpp(name="main")

    remote_path, _ = create_local_git_repo({"foo": "var"}, branch="my_release")

    client = TestClient()

    client.save({"conanfile.py": conan_file, "my_src/main.cpp": app,
                 "my_src/CMakeLists.txt": cmake,
                 ".gitignore": "build_*\n"})
    client.init_git_repo()
    client.run_command('git remote add origin "%s"' % remote_path.replace("\\", "/"))
    client.run_command('git push origin master')

    client.run("install . -if=install")
    client.run("build . -if=install")
    assert os.path.exists(os.path.join(client.current_folder, "build_Release", app_name))
    client.run("create . ")
    assert "Created package revision" in client.out


@pytest.mark.parametrize("no_copy_source", ["False", "True"])
def test_zip_download_with_subfolder(no_copy_source):
    """If we have a zip with the sources in a subfolder, specifying it in the self.folders.source
    will unzip in the base and will work both locally (conan build) or in the cache
    (exporting the sources)"""

    tmp = TestClient()  # Used only to save some files, sorry for the lazyness
    cmake = gen_cmakelists(appname="my_app", appsources=["main.cpp"])
    app = gen_function_cpp(name="main")
    tmp.save({"subfolder/main.cpp": app,
              "subfolder/CMakeLists.txt": cmake,
              "ignored_subfolder/ignored.txt": ""})
    zippath = os.path.join(tmp.current_folder, "my_sources.zip")
    zipdir(tmp.current_folder, zippath)

    conan_file = GenConanfile() \
        .with_import("import os") \
        .with_import("from conans import tools") \
        .with_import("from conan.tools.cmake import CMake") \
        .with_name("app").with_version("1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_generator("CMakeToolchain") \
        .with_class_attribute("no_copy_source={}".format(no_copy_source))

    conan_file = str(conan_file)
    conan_file += """
    def source(self):
        tools.get("http://fake_url/my_sources.zip")

    def layout(self):
        self.folders.source = "src"

    def build(self):
        assert os.path.exists(os.path.join(self.source_folder, "subfolder", "CMakeLists.txt"))
        assert os.path.exists(os.path.join(self.source_folder,
                                           "ignored_subfolder", "ignored.txt"))
        cmake = CMake(self)
        cmake.configure(build_script_folder="subfolder")
        cmake.build()
    """
    client = TestClient()
    client.save({"conanfile.py": conan_file})

    with mock.patch("conans.client.tools.net.download") as mock_download:
        def download_zip(*args, **kwargs):
            copy(zippath, os.getcwd())
        mock_download.side_effect = download_zip
        client.run("create . ")


@pytest.mark.parametrize("no_copy_source", ["False", "True"])
def test_zip_download_with_subfolder_new_tools(no_copy_source):
    """If we have a zip with the sources in a subfolder, specifying it in the self.folders.source
    will unzip in the base and will work both locally (conan build) or in the cache
    (exporting the sources)"""

    tmp = TestClient()  # Used only to save some files, sorry for the lazyness
    cmake = gen_cmakelists(appname="my_app", appsources=["main.cpp"])
    app = gen_function_cpp(name="main")
    tmp.save({"subfolder/main.cpp": app,
              "subfolder/CMakeLists.txt": cmake,
              "ignored_subfolder/ignored.txt": ""})
    zippath = os.path.join(tmp.current_folder, "my_sources.zip")
    zipdir(tmp.current_folder, zippath)

    conan_file = GenConanfile() \
        .with_import("import os") \
        .with_import("from conan.tools.files import get") \
        .with_import("from conan.tools.cmake import CMake") \
        .with_name("app").with_version("1.0") \
        .with_settings("os", "arch", "build_type", "compiler") \
        .with_generator("CMakeToolchain") \
        .with_class_attribute("no_copy_source={}".format(no_copy_source))

    conan_file = str(conan_file)
    conan_file += """
    def source(self):
        get(self, "http://fake_url/my_sources.zip")

    def layout(self):
        self.folders.source = "src"

    def build(self):
        assert os.path.exists(os.path.join(self.source_folder, "subfolder", "CMakeLists.txt"))
        assert os.path.exists(os.path.join(self.source_folder,
                                           "ignored_subfolder", "ignored.txt"))
        cmake = CMake(self)
        cmake.configure(build_script_folder="subfolder")
        cmake.build()
    """
    client = TestClient()
    client.save({"conanfile.py": conan_file})

    with mock.patch("conan.tools.files.files.download") as mock_download:
        def download_zip(*args, **kwargs):
            copy(zippath, os.getcwd())
        mock_download.side_effect = download_zip
        client.run("create . ")



import os
import textwrap

import patch_ng
import pytest
from bottle import static_file

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient, StoppableThreadBottle
from conans.util.files import save


class MockPatchset:
    apply_args = None

    def apply(self, strip=0, root=None, fuzz=False):
        self.apply_args = (root, strip, fuzz)
        return True


@pytest.fixture
def mock_patch_ng(monkeypatch):
    mock = MockPatchset()

    def mock_fromstring(string):
        mock.string = string
        return mock

    monkeypatch.setattr(patch_ng, "fromfile", lambda _: mock)
    monkeypatch.setattr(patch_ng, "fromstring", mock_fromstring)
    return mock


class TestConanToolFiles:

    def test_imports(self):
        conanfile = GenConanfile().with_import("from conan.tools.files import load, save, "
                                               "mkdir, download, get, ftp_download")
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("install .")

    def test_load_save_mkdir(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.files import load, save, mkdir

            class Pkg(ConanFile):
                name = "mypkg"
                version = "1.0"
                def source(self):
                    mkdir(self, "myfolder")
                    save(self, "./myfolder/myfile", "some_content")
                    assert load(self, "./myfolder/myfile") == "some_content"
            """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("source .")

    def test_download(self):
        http_server = StoppableThreadBottle()
        file_path = os.path.join(temp_folder(), "myfile.txt")
        save(file_path, "some content")

        @http_server.server.get("/myfile.txt")
        def get_file():
            return static_file(os.path.basename(file_path), os.path.dirname(file_path))

        http_server.run_server()

        profile = textwrap.dedent("""\
            [conf]
            tools.files.download:retry=1
            tools.files.download:retry_wait=0
            """)

        conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile
            from conan.tools.files import download

            class Pkg(ConanFile):
                name = "mypkg"
                version = "1.0"
                def source(self):
                    download(self, "http://localhost:{}/myfile.txt", "myfile.txt")
                    assert os.path.exists("myfile.txt")
            """.format(http_server.port))

        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.save({"profile": profile})
        client.run("create . -pr=profile")


def test_patch(mock_patch_ng):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.files import patch

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"

            def build(self):
                patch(self, patch_file='path/to/patch-file', patch_type='security')
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run('create .')

    assert os.path.exists(mock_patch_ng.apply_args[0])
    assert mock_patch_ng.apply_args[1:] == (0, False)
    assert 'mypkg/1.0: Apply patch (security)' in str(client.out)


@pytest.mark.parametrize("no_copy_source", [False, True])
def test_patch_real(no_copy_source):
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import patch, save, load

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"
            exports_sources = "*"
            no_copy_source = %s

            def layout(self):
                self.folders.source = "src"
                self.folders.build = "build"

            def source(self):
                save(self, "myfile.h", "//dummy contents")
                patch(self, patch_file="patches/mypatch_h", patch_type="security")
                self.output.info("SOURCE: {}".format(load(self, "myfile.h")))

            def build(self):
                save(self, "myfile.cpp", "//dummy contents")
                if self.no_copy_source:
                    patch_file = os.path.join(self.source_folder, "../patches/mypatch_cpp")
                else:
                    patch_file = "patches/mypatch_cpp"
                patch(self, patch_file=patch_file, patch_type="security",
                      base_path=self.build_folder)
                self.output.info("BUILD: {}".format(load(self, "myfile.cpp")))
        """ % no_copy_source)

    client = TestClient()
    patch_contents = textwrap.dedent("""\
        --- myfile.{ext}
        +++ myfile.{ext}
        @@ -1 +1 @@
        -//dummy contents
        +//smart contents
        """)
    client.save({"conanfile.py": conanfile,
                 "patches/mypatch_h": patch_contents.format(ext="h"),
                 "patches/mypatch_cpp": patch_contents.format(ext="cpp")})
    client.run('create .')
    assert "mypkg/1.0: Apply patch (security)" in client.out
    assert "mypkg/1.0: SOURCE: //smart contents" in client.out
    assert "mypkg/1.0: BUILD: //smart contents" in client.out

    # Test local source too
    client.run("install .")
    client.run("source .")
    assert "conanfile.py (mypkg/1.0): Apply patch (security)" in client.out
    assert "conanfile.py (mypkg/1.0): SOURCE: //smart contents" in client.out
    client.run("build .")
    assert "conanfile.py (mypkg/1.0): Apply patch (security)" in client.out
    assert "conanfile.py (mypkg/1.0): BUILD: //smart contents" in client.out


def test_apply_conandata_patches(mock_patch_ng):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.files import apply_conandata_patches

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.11.0"

            def layout(self):
                self.folders.source = "source_subfolder"

            def build(self):
                apply_conandata_patches(self)
        """)
    conandata_yml = textwrap.dedent("""
        patches:
          "1.11.0":
            - patch_file: "patches/0001-buildflatbuffers-cmake.patch"
            - patch_file: "patches/0002-implicit-copy-constructor.patch"
              patch_type: backport
              patch_source: https://github.com/google/flatbuffers/pull/5650
              patch_description: Needed to build with modern clang compilers.
          "1.12.0":
            - patch_file: "patches/0001-buildflatbuffers-cmake.patch"
    """)

    client = TestClient()
    client.save({'conanfile.py': conanfile,
                 'conandata.yml': conandata_yml})
    client.run('create .')

    assert mock_patch_ng.apply_args[0].endswith('source_subfolder')
    assert mock_patch_ng.apply_args[1:] == (0, False)

    assert 'mypkg/1.11.0: Apply patch (backport): Needed to build with modern' \
           ' clang compilers.' in str(client.out)

    # Test local methods
    client.run("install . -if=install")
    client.run("build . -if=install")

    assert 'conanfile.py (mypkg/1.11.0): Apply patch (backport): Needed to build with modern' \
           ' clang compilers.' in str(client.out)


def test_apply_conandata_patches_relative_base_path(mock_patch_ng):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.files import apply_conandata_patches

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.11.0"

            def layout(self):
                self.folders.source = "source_subfolder"

            def build(self):
                apply_conandata_patches(self)
        """)
    conandata_yml = textwrap.dedent("""
        patches:
          "1.11.0":
            - patch_file: "patches/0001-buildflatbuffers-cmake.patch"
              base_path: "relative_dir"
    """)

    client = TestClient()
    client.save({'conanfile.py': conanfile,
                 'conandata.yml': conandata_yml})
    client.run('create .')

    assert mock_patch_ng.apply_args[0].endswith(os.path.join('source_subfolder', "relative_dir"))
    assert mock_patch_ng.apply_args[1:] == (0, False)


def test_no_patch_file_entry():
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.files import apply_conandata_patches

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.11.0"

            def layout(self):
                self.folders.source = "source_subfolder"

            def build(self):
                apply_conandata_patches(self)
        """)
    conandata_yml = textwrap.dedent("""
        patches:
          "1.11.0":
            - wrong_entry: "patches/0001-buildflatbuffers-cmake.patch"
          "1.12.0":
            - patch_file: "patches/0001-buildflatbuffers-cmake.patch"
    """)

    client = TestClient()
    client.save({'conanfile.py': conanfile,
                 'conandata.yml': conandata_yml})
    client.run('create .', assert_error=True)

    assert "The 'conandata.yml' file needs a 'patch_file' or 'patch_string' entry for every patch" \
           " to be applied" in str(client.out)

def test_patch_string_entry(mock_patch_ng):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.files import apply_conandata_patches

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.11.0"

            def build(self):
                apply_conandata_patches(self)
        """)
    conandata_yml = textwrap.dedent("""
        patches:
          "1.11.0":
            - patch_string: mock patch data
              patch_type: string
    """)

    client = TestClient()
    client.save({'conanfile.py': conanfile,
                 'conandata.yml': conandata_yml})
    client.run('create .')

    assert os.path.exists(mock_patch_ng.apply_args[0])
    assert mock_patch_ng.apply_args[1:] == (0, False)
    assert 'mock patch data' == mock_patch_ng.string.decode('utf-8')
    assert 'mypkg/1.11.0: Apply patch (string)' in str(client.out)

def test_relate_base_path_all_versions(mock_patch_ng):
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.files import apply_conandata_patches

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"

            def layout(self):
                self.folders.source = "source_subfolder"

            def build(self):
                apply_conandata_patches(self)
        """)
    conandata_yml = textwrap.dedent("""
        patches:
          - patch_file: "patches/0001-buildflatbuffers-cmake.patch"
            base_path: "relative_dir"
    """)

    client = TestClient()
    client.save({'conanfile.py': conanfile,
                 'conandata.yml': conandata_yml})
    client.run('create .')

    assert mock_patch_ng.apply_args[0].endswith(os.path.join('source_subfolder', "relative_dir"))
    assert mock_patch_ng.apply_args[1:] == (0, False)


def test_export_conandata_patches(mock_patch_ng):
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import export_conandata_patches, load

        class Pkg(ConanFile):
            name = "mypkg"
            version = "1.0"

            def layout(self):
                self.folders.source = "source_subfolder"

            def export_sources(self):
                export_conandata_patches(self)

            def source(self):
                self.output.info(load(self, os.path.join(self.export_sources_folder, "patches/mypatch.patch")))
        """)
    conandata_yml = textwrap.dedent("""
        patches:
          - patch_file: "patches/mypatch.patch"
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .", assert_error=True)
    assert "conandata.yml not defined" in client.out
    # Empty conandata
    client.save({"conandata.yml": ""})
    client.run("create .", assert_error=True)
    assert "export_conandata_patches(): No patches defined in conandata" in client.out
    assert "ERROR: mypkg/1.0: Error in source() method" in client.out
    # wrong patches
    client.save({"conandata.yml": "patches: 123"})
    client.run("create .", assert_error=True)
    assert "conandata.yml 'patches' should be a list or a dict"  in client.out

    # No patch found
    client.save({"conandata.yml": conandata_yml})
    client.run("create .", assert_error=True)
    assert "No such file or directory" in client.out

    client.save({"patches/mypatch.patch": "mypatch!!!"})
    client.run("create .")
    assert "mypkg/1.0: mypatch!!!" in client.out

    conandata_yml = textwrap.dedent("""
        patches:
            "1.0":
                - patch_file: "patches/mypatch.patch"
    """)
    client.save({"conandata.yml": conandata_yml})
    client.run("create .")
    assert "mypkg/1.0: mypatch!!!" in client.out



import os

from conans.client.tools import environment_append
from conans.model.manifest import FileTreeManifest
from conans.test.utils.test_files import temp_folder
from conans.util.files import load, md5, save


class TestManifest:

    def test_tree_manifest(self):
        tmp_dir = temp_folder()
        files = {"one.ext": "aalakjshdlkjahsdlkjahsdljkhsadljkhasljkdhlkjashd",
                 "path/to/two.txt": "asdas13123",
                 "two.txt": "asdasdasdasdasdasd",
                 "folder/damn.pyc": "binarythings",
                 "folder/damn.pyo": "binarythings2",
                 "pythonfile.pyc": "binarythings3"}
        for filename, content in files.items():
            save(os.path.join(tmp_dir, filename), content)

        manifest = FileTreeManifest.create(tmp_dir)

        manifest.save(tmp_dir)
        readed_manifest = FileTreeManifest.load(tmp_dir)

        assert readed_manifest.time == manifest.time
        assert readed_manifest == manifest
        # Not included the pycs or pyo
        assert set(manifest.file_sums.keys()) == {"one.ext", "path/to/two.txt", "two.txt"}

        for filepath, md5readed in manifest.file_sums.items():
            content = files[filepath]
            assert md5(content) == md5readed

    def test_already_pyc_in_manifest(self):
        tmp_dir = temp_folder()
        save(os.path.join(tmp_dir, "man.txt"), "1478122267\nconanfile.pyc: "
                                               "2bcac725a0e6843ef351f4d18cf867ec\n"
                                               "conanfile.py: 2bcac725a0e6843ef351f4d18cf867ec\n"
                                               "conanfile.pyo: 2bcac725a0e6843ef351f4d18cf867ec\n")

        read_manifest = FileTreeManifest.loads(load(os.path.join(tmp_dir, "man.txt")))
        # Not included the pycs or pyo
        assert set(read_manifest.file_sums.keys()) == {"conanfile.py"}

    def test_special_chars(self):
        tmp_dir = temp_folder()
        save(os.path.join(tmp_dir, "conanmanifest.txt"), "1478122267\nsome: file.py: 123\n")
        read_manifest = FileTreeManifest.load(tmp_dir)
        assert read_manifest.file_sums["some: file.py"] == "123"


def test_pycache_included():
    tmp_dir = temp_folder()
    files = {"__pycache__/damn.py": "binarythings",
             "pythonfile.pyc": "binarythings3"}
    for filename, content in files.items():
        save(os.path.join(tmp_dir, filename), content)

    with environment_append({"CONAN_KEEP_PYTHON_FILES": "1"}):
        manifest = FileTreeManifest.create(tmp_dir)
    manifest = repr(manifest)
    assert "pythonfile.pyc" in manifest
    assert "__pycache__/damn.py" in manifest



import textwrap

from parameterized import parameterized

from conans.client.graph.graph import CONTEXT_BUILD, CONTEXT_HOST
from conans.model.profile import Profile
from conans.test.integration.graph.core.cross_build._base_test_case import CrossBuildingBaseTestCase


class ClassicProtocExampleBase(CrossBuildingBaseTestCase):
    """ There is an application that requires the protobuf library, and also
        build_requires the protoc executable to generate some files, but protoc
        also requires the protobuf library to build.

        Expected packages:
            * host_machine: application, protobuf
            * build_machine: protoc, protobuf
    """

    application = textwrap.dedent("""
        from conans import ConanFile

        class Protoc(ConanFile):
            name = "app"
            version = "testing"

            settings = "os"
            requires = "protobuf/testing@user/channel"

            def build_requirements(self):
                self.build_requires("protoc/testing@user/channel")

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
    """)

    def setUp(self):
        super(ClassicProtocExampleBase, self).setUp()
        self._cache_recipe(self.protobuf_ref, self.protobuf)
        self._cache_recipe(self.protoc_ref, self.protoc)
        self._cache_recipe(self.app_ref, self.application)


class ClassicProtocExample(ClassicProtocExampleBase):

    @parameterized.expand([(True, ), (False, )])
    def test_crossbuilding(self, xbuilding):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        if xbuilding:
            profile_build = Profile()
            profile_build.settings["os"] = "Build"
            profile_build.process_settings(self.cache)
        else:
            profile_build = None

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build,
                                       install=True)

        # Check HOST packages
        #   - application
        application = deps_graph.root.dependencies[0].dst
        self.assertEqual(len(application.dependencies), 2)
        self.assertEqual(application.conanfile.name, "app")
        self.assertEqual(application.context, CONTEXT_HOST)
        self.assertEqual(application.conanfile.settings.os, profile_host.settings['os'])
        if xbuilding:
            #   - application::deps_cpp_info:
            protobuf_host_cpp_info = application.conanfile.deps_cpp_info["protobuf"]
            self.assertEqual(protobuf_host_cpp_info.includedirs, ['protobuf-host'])
            self.assertEqual(protobuf_host_cpp_info.libdirs, ['protobuf-host'])
            self.assertEqual(protobuf_host_cpp_info.bindirs, ['protobuf-host'])
            with self.assertRaises(KeyError):
                _ = application.conanfile.deps_cpp_info["protoc"]

            #   - application::deps_env_info:
            protoc_env_info = application.conanfile.deps_env_info["protoc"]
            self.assertEqual(protoc_env_info.PATH, ['protoc-build'])
            self.assertEqual(protoc_env_info.OTHERVAR, 'protoc-build')

            protobuf_env_info = application.conanfile.deps_env_info["protobuf"]
            self.assertEqual(protobuf_env_info.PATH, ['protobuf-build'])
            self.assertEqual(protobuf_env_info.OTHERVAR, 'protobuf-build')

        #   - protobuf host
        protobuf_host = application.dependencies[0].dst
        self.assertEqual(protobuf_host.conanfile.name, "protobuf")
        self.assertEqual(protobuf_host.context, CONTEXT_HOST)
        self.assertEqual(protobuf_host.conanfile.settings.os, profile_host.settings['os'])

        # Check BUILD packages (default behavior changes if we use profile_build)
        #   - protoc
        protoc_build = application.dependencies[1].dst
        self.assertEqual(protoc_build.conanfile.name, "protoc")
        self.assertEqual(protoc_build.context, CONTEXT_BUILD if xbuilding else CONTEXT_HOST)
        self.assertEqual(str(protoc_build.conanfile.settings.os),
                         (profile_build if xbuilding else profile_host).settings['os'])

        #   - protobuf
        protobuf_build = protoc_build.dependencies[0].dst
        self.assertEqual(protobuf_build.conanfile.name, "protobuf")
        self.assertEqual(protoc_build.context, CONTEXT_BUILD if xbuilding else CONTEXT_HOST)
        self.assertEqual(str(protobuf_build.conanfile.settings.os),
                         (profile_build if xbuilding else profile_host).settings['os'])





import textwrap

from conan.tools.files.files import load_toolchain_args
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_toolchain_empty_config():
    client = TestClient(path_with_spaces=False)

    conanfile = GenConanfile().with_settings("os", "compiler", "build_type", "arch").\
        with_generator("BazelToolchain")

    client.save({"conanfile.py": conanfile})
    client.run("install .")

    config = load_toolchain_args(client.current_folder)
    assert not config


def test_toolchain_loads_config_from_profile():
    client = TestClient(path_with_spaces=False)

    profile = textwrap.dedent("""
    include(default)
    [conf]
    tools.google.bazel:configs=["test_config", "test_config2"]
    tools.google.bazel:bazelrc_path=/path/to/bazelrc
    """)

    conanfile = GenConanfile().with_settings("os", "compiler", "build_type", "arch").\
        with_generator("BazelToolchain")

    client.save({
        "conanfile.py": conanfile,
        "test_profile": profile
    })
    client.run("install . -pr=test_profile")

    config = load_toolchain_args(client.current_folder)
    assert config['bazel_configs'] == "test_config,test_config2"
    assert config['bazelrc_path'] == "/path/to/bazelrc"



import os

from conan.tools.files.files import save_toolchain_args
from conan.tools.gnu import Autotools
from conans.test.utils.mocks import ConanFileMock
from conans.test.utils.test_files import temp_folder


def test_source_folder_works():
    folder = temp_folder()
    os.chdir(folder)
    save_toolchain_args({
        "configure_args": "-foo bar",
        "make_args": "",
        "autoreconf_args": ""}
    )
    conanfile = ConanFileMock()
    conanfile.folders.set_base_install(folder)
    sources = "/path/to/sources"
    conanfile.folders.set_base_source(sources)
    autotools = Autotools(conanfile)
    autotools.configure(build_script_folder="subfolder")
    assert conanfile.command.replace("\\", "/") == '"/path/to/sources/subfolder/configure" -foo bar '

    autotools = Autotools(conanfile)
    autotools.configure()
    assert conanfile.command.replace("\\", "/") == '"/path/to/sources/configure" -foo bar '



import pytest
import platform
import textwrap

from conans.test.assets.pkg_cmake import pkg_cmake
from conans.test.utils.tools import TestClient


@pytest.mark.tool_cmake
@pytest.mark.tool_intel_oneapi
@pytest.mark.xfail(reason="Intel oneAPI Toolkit is not installed on CI yet")
@pytest.mark.skipif(platform.system() != "Linux", reason="Only for Linux")
class TestIntelCC:
    """Tests for Intel oneAPI C++/DPC++ compilers"""

    def test_intel_oneapi_and_dpcpp(self):
        self.client = TestClient()
        # Let's create a default hello/0.1 example
        files = pkg_cmake("hello", "0.1")
        self.client.save(files)
        intel_profile = textwrap.dedent("""
            [settings]
            os=Linux
            arch=x86_64
            arch_build=x86_64
            compiler=intel-cc
            compiler.mode=dpcpp
            compiler.version=2021.3
            compiler.libcxx=libstdc++
            build_type=Release
            [env]
            CC=dpcpp
            CXX=dpcpp
        """)
        self.client.save({"intel_profile": intel_profile})
        # Build in the cache
        self.client.run('create . --profile:build=intel_profile --profile:host=intel_profile')
        assert ":: initializing oneAPI environment ..." in self.client.out
        assert ":: oneAPI environment initialized ::" in self.client.out
        assert "Check for working CXX compiler: /opt/intel/oneapi/compiler/2021.3.0" \
               "/linux/bin/dpcpp -- works" in self.client.out
        assert "hello/0.1: Package " \
               "'5d42bcd2e9be3378ed0c2f2928fe6dc9ea1b0922' created" in self.client.out



# -*- coding: utf-8 -*-


import os

import pytest
import six

from conan.tools.files import replace_in_file, save, load
from conans.test.utils.mocks import MockConanfile
from conans.test.utils.test_files import temp_folder


@pytest.mark.skipif(six.PY2, reason="only Py3")
def test_save_and_load_encoding():
    conanfile = MockConanfile({})
    tmp = temp_folder()
    file_path = os.path.join(tmp, "file.txt")

    # By default utf-8 is used
    save(conanfile, file_path, "你很重，伙計")
    contents = load(conanfile, file_path)
    assert isinstance(contents, str)
    assert contents == "你很重，伙計"

    # But you can save bytes directly with an encoding
    save(conanfile, file_path, bytes("你很重，伙計", "utf-16"))
    contents = load(conanfile, file_path, encoding="utf-16")
    assert contents == "你很重，伙計"

    # And you can specify different encoding
    save(conanfile, file_path, "你很重，伙計", encoding="utf-16")
    contents = load(conanfile, file_path, encoding="utf-16")
    assert contents == "你很重，伙計"

    save(conanfile, file_path, "regular contents")
    contents = load(conanfile, file_path)
    assert contents == "regular contents"


@pytest.mark.skipif(six.PY2, reason="only Py3")
def test_replace_in_file():
    conanfile = MockConanfile({})
    tmp = temp_folder()
    file_path = os.path.join(tmp, "file.txt")

    # By default utf-8 is used
    save(conanfile, file_path, "你很重，伙計")
    replace_in_file(conanfile, file_path, "重", "0")
    contents = load(conanfile, file_path)
    assert contents == "你很0，伙計"

    # Replacing with other encodings is also possible
    save(conanfile, file_path, "Ö¼", encoding="cp1252")
    replace_in_file(conanfile, file_path, "¼", "0", encoding="cp1252")
    contents = load(conanfile, file_path, encoding="cp1252")
    assert contents == "Ö0"

    save(conanfile, file_path, bytes("Ö¼", "ISO-8859-1"))
    replace_in_file(conanfile, file_path, "¼", "0", encoding="ISO-8859-1")
    contents = load(conanfile, file_path, encoding="ISO-8859-1")
    assert contents == "Ö0"

    # Replacing utf-16 is also possible but using "utf-16LE" (without BOM) to search and replace
    # otherwise the "search" string is not found because it contains also a BOM (header)
    save(conanfile, file_path, bytes("你很重，伙計", "utf-16"))
    replace_in_file(conanfile, file_path, "重", "0", encoding="utf-16")
    contents = load(conanfile, file_path, encoding="utf-16")
    assert contents == "你很0，伙計"



import platform
import textwrap

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_extra_flags_via_conf():
    os_ = platform.system()
    os_ = "Macos" if os_ == "Darwin" else os_

    profile = textwrap.dedent("""
        [settings]
        os=%s
        compiler=gcc
        compiler.version=6
        compiler.libcxx=libstdc++11
        arch=armv8
        build_type=Release

        [conf]
        tools.build:cxxflags=["--flag1", "--flag2"]
        tools.build:cflags+=["--flag3", "--flag4"]
        tools.build:sharedlinkflags+=["--flag5"]
        tools.build:exelinkflags+=["--flag6"]
        tools.build:defines+=["DEF1", "DEF2"]
        """ % os_)
    client = TestClient()
    conanfile = GenConanfile().with_settings("os", "arch", "compiler", "build_type")\
        .with_generator("AutotoolsToolchain")
    client.save({"conanfile.py": conanfile,
                "profile": profile})
    client.run("install . --profile:build=profile --profile:host=profile")
    toolchain = client.load("conanautotoolstoolchain{}".format('.bat' if os_ == "Windows" else '.sh'))
    if os_ == "Windows":
        assert 'set "CPPFLAGS=%CPPFLAGS% -DNDEBUG -DDEF1 -DDEF2"' in toolchain
        assert 'set "CXXFLAGS=%CXXFLAGS% -O3 -s --flag1 --flag2"' in toolchain
        assert 'set "CFLAGS=%CFLAGS% -O3 -s --flag3 --flag4"' in toolchain
        assert 'set "LDFLAGS=%LDFLAGS% --flag5 --flag6"' in toolchain
    else:
        assert 'export CPPFLAGS="$CPPFLAGS -DNDEBUG -DDEF1 -DDEF2"' in toolchain
        assert 'export CXXFLAGS="$CXXFLAGS -O3 -s --flag1 --flag2"' in toolchain
        assert 'export CFLAGS="$CFLAGS -O3 -s --flag3 --flag4"' in toolchain
        assert 'export LDFLAGS="$LDFLAGS --flag5 --flag6"' in toolchain


def test_not_none_values():

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.gnu import AutotoolsToolchain

        class Foo(ConanFile):
            name = "foo"
            version = "1.0"

            def generate(self):
                tc = AutotoolsToolchain(self)
                assert None not in tc.defines
                assert None not in tc.cxxflags
                assert None not in tc.cflags
                assert None not in tc.ldflags

    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("install .")




import sys
import unittest

import six
import pytest

from conans.test.utils.tools import TestClient


class PythonVersionTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()

    def _validate_message(self, expected_message):
        self.client.run("--help")
        self.assertIn(expected_message, str(self.client.out))

    @pytest.mark.skipif(not six.PY2, reason="Requires Python 2.7")
    def test_py2_warning_message(self):
        self._validate_message("Python 2 is deprecated as of 01/01/2020 and Conan has stopped\n"
                               "supporting it officially")

    @pytest.mark.skipif(sys.version_info.major != 3 or sys.version_info.minor != 4, reason="Requires Python 3.4")
    def test_py34_warning_message(self):
        self._validate_message("Python 3.4 support has been dropped. It is strongly "
                               "recommended to use Python >= 3.5 with Conan")



import unittest
from parameterized import parameterized
from textwrap import dedent

from conans.test.utils.tools import TestClient


class CppStdMinimumVersionTests(unittest.TestCase):

    CONANFILE = dedent("""
        import os
        from conans import ConanFile
        from conans.tools import check_min_cppstd, valid_min_cppstd

        class Fake(ConanFile):
            name = "fake"
            version = "0.1"
            settings = "compiler"

            def configure(self):
                check_min_cppstd(self, "17", False)
                self.output.info("valid standard")
                assert valid_min_cppstd(self, "17", False)
        """)

    PROFILE = dedent("""
        [settings]
        compiler=gcc
        compiler.version=9
        compiler.libcxx=libstdc++
        {}
        """)

    def setUp(self):
        self.client = TestClient()
        self.client.save({"conanfile.py": CppStdMinimumVersionTests.CONANFILE})

    @parameterized.expand(["17", "gnu17"])
    def test_cppstd_from_settings(self, cppstd):
        profile = CppStdMinimumVersionTests.PROFILE.replace("{}", "compiler.cppstd=%s" % cppstd)
        self.client.save({"myprofile":  profile})
        self.client.run("create . user/channel -pr myprofile")
        self.assertIn("valid standard", self.client.out)

    @parameterized.expand(["11", "gnu11"])
    def test_invalid_cppstd_from_settings(self, cppstd):
        profile = CppStdMinimumVersionTests.PROFILE.replace("{}", "compiler.cppstd=%s" % cppstd)
        self.client.save({"myprofile": profile})
        self.client.run("create . user/channel -pr myprofile", assert_error=True)
        self.assertIn("Invalid configuration: Current cppstd (%s) is lower than the required C++ "
                      "standard (17)." % cppstd, self.client.out)



import platform
import textwrap

import pytest

from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_msbuild_deps_components():
    # TODO: Duplicated from xcodedeps_components
    client = TestClient()

    client.run("new tcp/1.0 -m=cmake_lib")
    client.run("create . -tf=None")

    header = textwrap.dedent("""
        #pragma once
        void {name}();
        """)

    source = textwrap.dedent("""
        #include <iostream>
        #include "{name}.h"
        {include}

        void {name}(){{
            {call}
            #ifdef NDEBUG
            std::cout << "{name}/1.0: Hello World Release!" << std::endl;
            #else
            std::cout << "{name}/1.0: Hello World Debug!" << std::endl;
            #endif
        }}
        """)

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(myproject CXX)

        find_package(tcp REQUIRED CONFIG)

        add_library(core src/core.cpp include/core.h)
        add_library(client src/client.cpp include/client.h)
        add_library(server src/server.cpp include/server.h)

        target_include_directories(core PUBLIC include)
        target_include_directories(client PUBLIC include)
        target_include_directories(server PUBLIC include)

        set_target_properties(core PROPERTIES PUBLIC_HEADER "include/core.h")
        set_target_properties(client PROPERTIES PUBLIC_HEADER "include/client.h")
        set_target_properties(server PROPERTIES PUBLIC_HEADER "include/server.h")

        target_link_libraries(client core tcp::tcp)
        target_link_libraries(server core tcp::tcp)

        install(TARGETS core client server)
        """)

    conanfile_py = textwrap.dedent("""
        import os

        from conan import ConanFile
        from conan.tools.cmake import CMake, cmake_layout


        class LibConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "CMakeLists.txt", "src/*", "include/*"
            {requires}
            generators = "CMakeToolchain", "CMakeDeps"

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()

            def package(self):
                cmake = CMake(self)
                cmake.install()
            {package_info}
        """)

    network_pi = """
    def package_info(self):
        self.cpp_info.components["core"].libs = ["core"]
        self.cpp_info.components["core"].includedirs.append("include")
        self.cpp_info.components["core"].libdirs.append("lib")

        self.cpp_info.components["client"].libs = ["client"]
        self.cpp_info.components["client"].includedirs.append("include")
        self.cpp_info.components["client"].libdirs.append("lib")
        self.cpp_info.components["client"].requires.extend(["core", "tcp::tcp"])

        self.cpp_info.components["server"].libs = ["server"]
        self.cpp_info.components["server"].includedirs.append("include")
        self.cpp_info.components["server"].libdirs.append("lib")
        self.cpp_info.components["server"].requires.extend(["core", "tcp::tcp"])
    """

    client.save({
        "include/core.h": header.format(name="core"),
        "include/server.h": header.format(name="server"),
        "include/client.h": header.format(name="client"),
        "src/core.cpp": source.format(name="core", include="", call=""),
        "src/server.cpp": source.format(name="server",
                                        include='#include "core.h"\n#include "tcp.h"',
                                        call="core(); tcp();"),
        "src/client.cpp": source.format(name="client", include='#include "core.h"\n#include "tcp.h"',
                                        call="core(); tcp();"),
        "conanfile.py": conanfile_py.format(requires='requires= "tcp/1.0"', package_info=network_pi),
        "CMakeLists.txt": cmakelists,
    }, clean_first=True)

    client.run("create . network/1.0@")

    chat_pi = """
    def package_info(self):
        self.cpp_info.libs = ["chat"]
        self.cpp_info.includedirs.append("include")
        self.cpp_info.libdirs.append("lib")
        self.cpp_info.requires.append("network::client")
    """

    cmakelists_chat = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(chat CXX)
        find_package(network REQUIRED CONFIG)
        add_library(chat src/chat.cpp include/chat.h)
        target_include_directories(chat PUBLIC include)
        set_target_properties(chat PROPERTIES PUBLIC_HEADER "include/chat.h")
        target_link_libraries(chat network::client)
        install(TARGETS chat)
        """)

    client.save({
        "include/chat.h": header.format(name="chat"),
        "src/chat.cpp": source.format(name="chat", include='#include "client.h"', call="client();"),
        "conanfile.py": conanfile_py.format(requires='requires= "network/1.0"',
                                            package_info=chat_pi),
        "CMakeLists.txt": cmakelists_chat,
    }, clean_first=True)

    client.run("create . chat/1.0@")

    client.run("new greet/0.1 --template=msbuild_exe")
    conanfile = client.load("conanfile.py").replace("settings = ",
                                                    'requires="chat/1.0"\n'
                                                    '    generators = "MSBuildDeps"\n'
                                                    '    settings = ')
    vcproj = client.load("greet.vcxproj")
    vcproj2 = vcproj.replace(r'<Import Project="conan\\conantoolchain.props" />',
                             r"""<Import Project="conan\\conantoolchain.props" />
                               <Import Project="conan\\conandeps.props" />
                             """)
    assert vcproj2 != vcproj
    client.save({"conanfile.py": conanfile,
                 "src/greet.cpp": gen_function_cpp(name="main", includes=["chat"],
                                                   calls=["chat"]),
                 "greet.vcxproj": vcproj2})
    client.run("create .")
    assert "main: Release!" in client.out
    assert "core/1.0: Hello World Release!" in client.out
    assert "tcp/1.0: Hello World Release!" in client.out
    assert "client/1.0: Hello World Release!" in client.out
    assert "chat/1.0: Hello World Release!" in client.out
    assert "server/1.0" not in client.out



# coding=utf-8

import unittest

import pytest

from conans import Settings
from conans.client.conf import get_default_settings_yml
from conans.client.settings_preprocessor import preprocess
from conans.errors import ConanException


class SettingsCompilerIntelVisualPreprocessorTest(unittest.TestCase):

    def setUp(self):
        self.settings = Settings.loads(get_default_settings_yml())
        self.settings.compiler = "intel"
        self.settings.compiler.base = "Visual Studio"

    def test_release_build_type_runtime(self):
        self.settings.build_type = "Release"
        preprocess(self.settings)
        self.assertEqual(self.settings.compiler.base.runtime, "MD")

    def test_debug_build_type_runtime(self):
        self.settings.build_type = "Debug"
        preprocess(self.settings)
        self.assertEqual(self.settings.compiler.base.runtime, "MDd")

    def test_different_base_compiler(self):
        self.settings.compiler.base = "gcc"
        self.settings.build_type = "Debug"
        preprocess(self.settings)
        self.assertIsNone(self.settings.compiler.base.get_safe("runtime"))

    def test_custom_base_runtime_set(self):
        self.settings.build_type = "Debug"
        self.settings.compiler.base.runtime = "MT"
        preprocess(self.settings)
        self.assertEqual(self.settings.compiler.base.runtime, "MT")


class SettingsCompilerVisualPreprocessorTest(unittest.TestCase):

    def setUp(self):
        self.settings = Settings.loads(get_default_settings_yml())
        self.settings.compiler = "Visual Studio"

    def test_release_build_type_runtime(self):
        self.settings.build_type = "Release"
        preprocess(self.settings)
        self.assertEqual(self.settings.compiler.runtime, "MD")

    def test_debug_build_type_runtime(self):
        self.settings.build_type = "Debug"
        preprocess(self.settings)
        self.assertEqual(self.settings.compiler.runtime, "MDd")

    def test_different_base_compiler(self):
        self.settings.compiler = "gcc"
        self.settings.build_type = "Debug"
        preprocess(self.settings)
        self.assertIsNone(self.settings.compiler.get_safe("runtime"))

    def test_custom_base_runtime_set(self):
        self.settings.build_type = "Debug"
        self.settings.compiler.runtime = "MT"
        preprocess(self.settings)
        self.assertEqual(self.settings.compiler.runtime, "MT")


class TestSettingsCompilerMSVCPreprocessorTest:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.settings = Settings.loads(get_default_settings_yml())
        self.settings.compiler = "msvc"

    def test_release_build_type_runtime(self):
        self.settings.build_type = "Release"
        preprocess(self.settings)
        assert self.settings.compiler.runtime_type == "Release"

    def test_debug_build_type_runtime(self):
        self.settings.build_type = "Debug"
        preprocess(self.settings)
        assert self.settings.compiler.runtime_type == "Debug"

    def test_different_base_compiler(self):
        self.settings.compiler = "gcc"
        self.settings.build_type = "Debug"
        preprocess(self.settings)
        assert self.settings.compiler.get_safe("runtime") is None

    def test_custom_base_runtime_set(self):
        self.settings.build_type = "Debug"
        with pytest.raises(ConanException) as e:
            self.settings.compiler.runtime = "MT"
            preprocess(self.settings)
        assert "Invalid setting 'MT' is not a valid 'settings.compiler.runtime' value" \
               in str(e.value)
